/*==============================================================================
#  $Id: createRecorded.c,v 1.3 2003/10/03 22:07:30 moyekj Exp $
# Copyright (C) 2003 Kevin J. Moye <moyekj@yahoo.com>
#
# Author: Kevin J. Moye
# With contributions by Philip Van Baren
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
==============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include "replaySchedule.h"

#define MAX_UNIQUE_CAT 40

// Populate show category names based on information from channel structures
void defineCategoryNames(
   struct Show **shows, int numShow, struct Channel **channels, int numChan)
{
 int i,j;
   for(i=0; i<numShow; ++i) {
      for(j=0; j<numChan; ++j) {
         if( channels[j]->channelID == shows[i]->channelID ) {
            strcpy(shows[i]->CategoryName,channels[j]->CategoryName);
         }
      }
   }
}

// Build html category link block
char *buildCatLinks( struct Show *show, char **unique, int numCat )
{
 static char line[MAXLINE];
 char string[MAXLINE];
 int i;

   strcpy(line,"<b>Categories:</b>&nbsp;");
   for( i=0; i<numCat; ++i ) {
      if( !strcmp(show->CategoryName,unique[i]) ) {
         sprintf(string, "<A NAME=\"%s\"><b>%s</b></A>",unique[i], unique[i]);
      } else {
         sprintf(string, "<A HREF=\"#%s\">%s</A>", unique[i], unique[i]);
      }
      strcat(line," ");
      strcat(line,string);
   }
   
   return(line);
}

// Sort recorded shows by category name
void sortShows(
   struct Show ***sortedShows, struct Show ***shows, int numShows,
   struct Channel **channels, int numChan, char ***unique, int *numCat )
{
int i, j, *order=NULL, num=0;
char done[2*MAXLINE];
char cat[256];

   order = (int *)malloc(sizeof(int)*numShows);
   (*sortedShows) = (struct Show **)malloc(sizeof(struct Show *)*numShows);
   strcpy(done,"");

   // Pull out all unique category names
   for( i=0; i<numShows; ++i ) {
      strcpy(cat, (*shows)[i]->CategoryName);
      if( !strstr(done,strcat(cat,",")) ) {
         strcpy((*unique)[(*numCat)++],(*shows)[i]->CategoryName);
         strcat(done,cat);
      }
   }

   // Initialize order list
   for( i=0; i<numShows; ++i ) {
      order[i] = -1;
   }

   // Build order list
   while( num < numShows ) {
      for( i=0; i<(*numCat); ++i ) {
         for( j=0; j<numShows; ++j ) {
            if( !strcmp((*shows)[j]->CategoryName,(*unique)[i]) ) {
               if( order[num] == -1 )
                  order[num++] = j;
            }
         }
      }
   }

   // Create new sorted order of programs
   for( i=0; i<num; ++i ) {
      (*sortedShows)[i] = (struct Show *)malloc(sizeof(struct Show *));
      (*sortedShows)[i] = (*shows)[order[i]];
   }

   if(order) free(order);
}

// Create html file displaying already recorded shows
int createRecordedSchedule(
   char htmlFile[], char replayIP[MAX_IP_NUM][256], int ipCount,
   struct Show **shows, int numShows,
   struct Channel **channels, int numChan)
{
FILE *HTML;
struct Time today, start, stop;
struct Show *s;
char string[MAXLINE], s1[50], s2[50];
int  i, skip;
time_t now;
struct Show **sortedShows=NULL;
char **unique=NULL, name[256];
int  numCat=0;

  if(!quiet) fprintf(stderr,">> Creating html file: '%s' ...\n", htmlFile);

  // Add category names to shows
  defineCategoryNames(shows, numShows, channels, numChan);

  // Sort recorded shows by category name
  unique = (char **)malloc(sizeof(char)*MAX_UNIQUE_CAT*256);
  for(i=0; i<MAX_UNIQUE_CAT; ++i) {
     unique[i] = (char *)malloc(sizeof(char)*256);
  }
  sortShows(
     &sortedShows, &shows, numShows, channels, numChan, &unique, &numCat
  );

  HTML = fopen(htmlFile, "w");
  if( ! HTML ) {
     fprintf(stderr, "*ERROR* Failed to write to file: '%s'\n", htmlFile);
     return(0);
  }

  now = time(NULL);
  today = utcToLocalTime(now);
  sprintf(string, "Recorded Shows on Replay(s):");
  for(i=0; i<ipCount; ++i) {
     strcat(string, "&nbsp;\"");
     strcat(string, getReplayName(replayIP[i]));
     strcat(string, "\"");
     if( i != ipCount-1 )
        strcat(string, ",");
  }
  fprintf(HTML, "<!DOCTYPE HTML PUBLIC>\n");
  fprintf(HTML, "<html><body bgcolor=\"%s\">\n", CONFIG.htmlBackgroundColor);
  fprintf(HTML, "<title>%s</title>\n",string);
  fprintf(HTML, "<P align=\"left\">\n");
  fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
  fprintf(HTML, "<b>Today is: %s %s %d %d</b>\n", today.wday, today.month, today.mday, today.year);
  fprintf(HTML, "<br>\n");
  fprintf(HTML, "<b>%s</b></font>&nbsp;&nbsp;",string);
  fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
  strcpy(name,"");
  for( i=0; i<numShows; ++i ) {
     s = sortedShows[i];
     skip = 0;
     start = utcToLocalTime(s->ustart);
     stop = utcToLocalTime(s->ustop);

     if( ! skip ) {
      // Category Links block
      if ( strcmp(s->CategoryName,name) ) {
         strcpy(string, buildCatLinks(s,unique,numCat));
         fprintf(HTML, "<br><br><br><br>\n");
         fprintf(HTML, "%s\n",string);
         fprintf(HTML, "<br>\n");
         strcpy(name, s->CategoryName);
      }

      // Show info block
      fprintf(HTML, "<hr width=\"100%%\">\n");
      fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
      if( ipCount > 1 ) fprintf(HTML, "%s\n<br>\n", getReplayName(s->ip));
      fprintf(HTML, "%d (%s)&nbsp;&nbsp;&nbsp;<b>%s</b>",
         s->ChannelNum, s->ChannelName, s->Title
      );
      if( strlen(s->Episode) > 0 ) {
         fprintf(HTML, " - %s", s->Episode);
      }
      fprintf(HTML, "<br>\n");
      sprintf(s1, "%d:%d", start.hour, start.min);
      sprintf(s2, "%d:%d", stop.hour, stop.min);
      sprintf(string, formatTimeRange(s1,s2));
      fprintf(HTML, "%s\n",string);
      fprintf(HTML, " %s %s %d %d\n",
         start.wday, start.month, start.mday, start.year
      );
      fprintf(HTML, "\n<br>\n");
      fprintf(HTML, "%s\n",s->Description);
      if( strlen(s->Actors) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "%s\n",s->Actors);
      }
      if( strlen(s->Guests) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "%s\n",s->Guests);
      }
      if( strlen(s->Producers) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "Produced by %s\n",s->Producers);
      }
      if( strlen(s->Directors) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "Directed by %s\n",s->Directors);
      }
      if( strlen(s->OtherInfo) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "%s\n",s->OtherInfo);
      }
     }
   }

   // All done with html
   fprintf(HTML, "</body></html>\n");
   fclose(HTML);

   // Free memory
   for(i=0; i<MAX_UNIQUE_CAT; i++) {
      free(unique[i]);
   }
   free(unique);

   if(sortedShows) free(sortedShows);

   return(1);
}
