#!/usr/bin/perl -w
# Perl script to download TV listings using DataDirect service (via subscription)

### START MAIN ###
use POSIX qw( strftime );
use CGI qw(:standard);
use English;
use Time::Local;
use SOAP::Lite; 
use Unicode::String;
require LWP::Simple;
require LWP::UserAgent;
require HTTP::Request;
require HTTP::Headers;

Unicode::String->stringify_as( 'latin1' );

$webservice = "http://docs.tms.tribune.com/tech/tmsdatadirect/zap2it/xtvd.wsdl";
$username = "username";			# Username for DataDirect
$password = "password";			# Password for DataDirect
$days = 8;				# Default Days to Download
$verbose = 1;				# Verbose Output
$cnf_xmlfile = "./dd.xml";		# XML Output File

if ( -f $cnf_xmlfile ) {
   rename($cnf_xmlfile, "$cnf_xmlfile.old");
}

$now = strftime( "%Y-%m-%d", localtime ) . " " . strftime( "%H:%M:%S", localtime );
$started = time;

$starttime_seconds = &ddStartTime();
$endtime_seconds = $starttime_seconds + ((($days * 24) * 60) * 60);
$starttime = &ddTime($starttime_seconds);
$endtime = &ddTime($endtime_seconds);

&vprint("Requesting $days days of data from DataDirect");
&vprint("($starttime -> $endtime)");
&vprint("Opening connection to: $webservice");
$soapenv = SOAP::Lite
 -> service($webservice)
 -> outputxml('true')
 -> on_fault( sub { 
	my($soap,$res)=@_; 
	print "SOAP call failed: ". 
	(ref $res ? $res->faultstring : $soap->transport->status)."\n"; 
	exit 1; 
	} );

&vprint("Downloading Data... ");
$xtvddoc = $soapenv
  -> proxy('http://notused/', options => {compress_threshold => 10000}, timeout => 420)
  -> download("<startTime>$starttime</startTime><endTime>$endtime</endTime>");

&vprint("UTF Encoding... ");
$xtvddoc = Unicode::String::utf8( $xtvddoc );

$accessdenied = 0;
$message = "";

if ($xtvddoc =~ "<HTML><HEAD><TITLE>401") {
	print "DataDirect: Access Denied\n";
	$accessdenied = 1;
	$xtvddoc = "";
}

if ($xtvddoc =~ "<HTML><HEAD><TITLE>500") {
	print "DataDirect: Server Error\n";
	$message = $xtvddoc;
	$message =~ s/.*<PRE>([^<]*)\W.*/$1/;
	if ($message eq $xtvddoc) {
		$message = "";
	}
	$xtvddoc = "";
}

if ($xtvddoc =~ "<HTML><HEAD><TITLE>") {
	print "DataDirect: Unknown Error\n";
	$xtvddoc = "";
}

if ($xtvddoc =~ m#(<xtvd\s.*</xtvd>)#s) {
	&vprint("Writing Data to $cnf_xmlfile... ");

	open(HOUTPUT,">$cnf_xmlfile");
		binmode HOUTPUT;
		print HOUTPUT $xtvddoc;
	close HOUTPUT;

	&vprint("Done!\n");

	$exitflag = 0;

	print "Messages from DataDirect:\n";

	open(HINPUT,"<$cnf_xmlfile");
	do {
		$line = readline HINPUT;
		chomp($line);
		if ($line =~ /message/) {
			$message = $line;
			$message =~ s/.*<message>([^<]*)\W.*/$1/;
			if ($message eq $line) {
					$message = "";
			}
		}
		if ($line =~ /faultstring/) {
			$message = $line;
			$message =~ s/.*<faultstring>([^<]*)\W.*/$1/;
			if ($message eq $line) {
					$message = "";
			}
			$exitflag = 1;
		}

		if ($line =~ /<\/messages>/) {
			$exitflag = 1;
		}
		if ($line =~ /<messages\/>/) {
			$message = "None";
			$exitflag = 1;
		}
		if (length($message) > 0) {
			print "$message\n";
			$message = "";
		}
	} until ($exitflag > 0);

	close HINPUT;

}else{
	if (!$accessdenied) {
		if (length($message) > 0) {
			print "\n$message\n";
		}else{
			print "Error: $xtvddoc\n";
		}
	}
}

$finished = time;
$runtime = $finished - $started;
$now = strftime( "%Y-%m-%d", localtime ) . " " . strftime( "%H:%M:%S", localtime );

&vprint("\nJob finished at $now ($runtime seconds)");
### END MAIN ###

# Return UTC
sub ddStartTime {
   my ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isdst) = localtime();
   $sec = 0;
   $min = 0;
   $hour = 0;
   my $utc = timelocal($sec, $min, $hour, $mday, $month, $year);
   return($utc);
}

# DataDirect start time format is: yyyy-MM-ddThh:mm:ssZ
sub ddTime {
 my $utc = shift;
   my ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isdst) = gmtime($utc);
   $year += 1900;
   $month += 1;
   my $t = sprintf("%4d-%02d-%02dT%02d:%02d:%02dZ",$year,$month,$mday,$hour,$min,$sec);
   return($t);
}

sub vprint {
 my $string = shift;
   if ($verbose) {
      print "$string\n";
   }
}

sub SOAP::Transport::HTTP::Client::get_basic_credentials {
   return "$username" => "$password";
}
