#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// match(buffer, word)
// Compare two strings, ignoring leading whitespace
// returns NULL if not equal
// returns pointer to the following data in first string if eqal
char *match(char *buffer, char *word) {
    int i,j;
    for(i=0; buffer[i] && isspace(buffer[i]); i++);
    for(j=0; buffer[i] && word[j] && (buffer[i]==word[j]); i++, j++);
    if(word[j])
	return NULL;
    // Skip to the beginning of the following word, or end of line
    for( ; buffer[i] && isspace(buffer[i]); i++);
    return buffer+i;
}

// skipword(buffer)
// skip over any whitespace-word-whitespace strings
// and return a pointer to the following data
// returns NULL if the end of line is reached
char *skipword(char *buffer) {
    int i;
    for(i=0; buffer[i] &&  isspace(buffer[i]); i++);
    for(   ; buffer[i] && !isspace(buffer[i]); i++);
    for(   ; buffer[i] &&  isspace(buffer[i]); i++);
    if(!buffer[i])
	return NULL;
    else
	return buffer+i;
}

#define MAX_LINE_LENGTH 1000
#define MAX_BLOCK_NEST 10
#define MAX_BLOCK_NAME_LENGTH 20

struct channel_info {
    unsigned int modification_time;
    unsigned int file_number;
    unsigned int start_offset;
    unsigned int end_offset;
    unsigned int line_count;
    unsigned int min_start;
    unsigned int max_start;
};

// Maximum number of channel entries we will handle
// Really should dynamically re-allocate, but I am lazy...
#define MAX_CHANNELS 2000
#define MAX_DAYS 20

int main(int argc, char *argv[])
{
    int delete_old_files=1;   // Set TRUE if you want to automatically remove outdated files

    int i,j,k,file,line_count,blocknest,channelcount;
    char *ptr;
    char buffer[MAX_LINE_LENGTH];
    char blocklist[MAX_BLOCK_NEST][MAX_BLOCK_NAME_LENGTH];
    unsigned int min_start,max_start,modification_time,tmsid;
    unsigned int start_offset,end_offset;
    unsigned int starttime;
    unsigned int tmsid_map[MAX_CHANNELS];
    struct channel_info *info[MAX_CHANNELS];
    unsigned int channel_tuning[MAX_CHANNELS];
    char *channel_name[MAX_CHANNELS];
    unsigned int tuning;
    char name[MAX_LINE_LENGTH];
    int has_headend = 0;
    FILE *fp;
    char **filelist;
    int filecount;
    unsigned int now=time(NULL);

    if(argc<1) {
        fprintf(stderr,"Usage: %s guide1.asc guide2.asc ... guideN.asc\n",argv[0]);
        exit(0);
    }
    blocknest = 0;
    channelcount = 0;
    // Scan the list of files
    filelist = argv+1;
    filecount = argc-1;
    for(file = 0; file<filecount; file++) {
	fp = fopen(filelist[file], "r");
	if(fp == NULL) {
	    fprintf(stderr,"ERROR: Could not open '%s'\n",filelist[file]);
	    exit(1);
	}
        fprintf(stderr,"Scanning '%s' ...\n",filelist[file]);
	while(!feof(fp)) {
	    fgets(buffer,sizeof(buffer),fp);
	    // Remove trailing whitespace
	    for(i=0; buffer[i]; i++);
	    for(i--; (i>=0) && isspace(buffer[i]); buffer[i--]=0);
	    line_count++;
	    if(strchr(buffer, '{') && (blocknest < MAX_BLOCK_NEST)) {
		// Push the block type less leading and trailing spaces
		for(i=0; buffer[i] && isspace(buffer[i]); i++);
		for(j=0; buffer[i] && (buffer[i] != '{') && (j<MAX_BLOCK_NAME_LENGTH); i++,j++)
		    blocklist[blocknest][j] = buffer[i];
		for(j-- ; (j>=0) && isspace(blocklist[blocknest][j]); j--)
		    blocklist[blocknest][j] = 0;
		// Reset the program details when starting a channel lineup
		if(strcmp(blocklist[blocknest], "Channel Lineup")==0) {
		    start_offset = ftell(fp);
		    line_count = 0;
		    min_start = (unsigned)(-1);
		    max_start = 0;
		    modification_time = 0;
		    tmsid = 0;
		}
		if(strcmp(blocklist[blocknest], "Headend Channel")==0) {
		    // Reset the channel info
		    tmsid = 0;
		    tuning = 0;
		    name[0] = 0;
		    has_headend = 1;
		}
		//fprintf(stderr,"Start block: '%s'\n", blocklist[blocknest]);
		blocknest++;
	    } else if(ptr = match(buffer, "Modification Time")) {
		modification_time = atol(ptr);
	    } else if((ptr = match(buffer, "TMS ID"))
		&& (ptr = skipword(ptr)) ) {
		tmsid = atol(ptr);
	    } else if((ptr = match(buffer, "Tuning"))
		&& (ptr = skipword(ptr)) ) {
		tuning = atol(ptr);
	    } else if(ptr = match(buffer, "Name")) {
		strcpy(name,ptr);
	    } else if(ptr = match(buffer, "Event Time")) {
		starttime = atol(ptr);
		if(starttime < min_start)
		    min_start = starttime;
		if(starttime > max_start)
		    max_start = starttime;
	    } else if((blocknest > 0) && match(buffer, "}") ) {
		// Pop the block type
		//fprintf(stderr, "End block: '%s'\n", blocklist[blocknest-1]);
		blocknest--;
		if((strcmp(blocklist[blocknest],"Headend Channel") == 0) && (tmsid != 0)) {
		    // Add this channel to the channel list
		    if(channelcount < MAX_CHANNELS) {
			// New channel - allocate a new structure
			//fprintf(stderr, "  New channel: 0x%08x\n", tmsid);
			tmsid_map[channelcount] = tmsid;
			channel_tuning[channelcount] = tuning;
			channel_name[channelcount] = strdup(name);
			info[channelcount] = (struct channel_info *)malloc(MAX_DAYS * sizeof(struct channel_info));
			memset(info[channelcount],0,MAX_DAYS * sizeof(struct channel_info));
			channelcount++;
		    } else {
			fprintf(stderr,"ERROR: Reached the hard-coded channel count limit: %d\n",MAX_CHANNELS);
		    }
		}
		if((strcmp(blocklist[blocknest],"Channel Lineup") == 0) && (tmsid != 0)) {
		    if(max_start < (now-86400)) {
			fprintf(stderr,"   File '%s' has old channel guide info, skipping...\n",filelist[file]);
			break;
		    }
		    end_offset = ftell(fp);
		    //fprintf(stderr, "  0x%08x %u %u\n",tmsid, min_start, max_start);
		    // Find this TMS ID in the channel list
		    for(i=0; 
			(i<channelcount) && (tmsid_map[i] != tmsid);
			i++);
		    if((i == channelcount) && !has_headend) {
			if(channelcount >= MAX_CHANNELS) {
			    fprintf(stderr,"ERROR: Reached the hard-coded channel count limit: %d\n",MAX_CHANNELS);
			} else {
			    // New channel - allocate a new structure
			    //fprintf(stderr, "  New channel: 0x%08x\n", tmsid);
		 	    tmsid_map[i] = tmsid;
			    info[i] = (struct channel_info *)malloc(MAX_DAYS * sizeof(struct channel_info));
			    channel_name[i] = NULL;
			    channel_tuning[i] = 0;
			    memset(info[i],0,MAX_DAYS * sizeof(struct channel_info));
			    channelcount++;
			}
		    }
		    if(i < channelcount) {
		        // Search this channel for duplicate times
		        for(j=0; 
			       (j<MAX_DAYS)
			    && (info[i][j].modification_time != 0)
			    && (   (info[i][j].min_start >= max_start)
			        || (info[i][j].max_start <= min_start));
 			    j++);
		        // Save this info off to the array of channel information
		        if(j < MAX_DAYS) {
			    // Keep the most recent data
			    // If we found modification_time==0, then this is a new entry
			    if(info[i][j].modification_time < modification_time) {
			        /*if(info[i][j].modification_time != 0) {
				    fprintf(stderr,"  Overriding outdated guide info for 0x%08x (%u < %u)\n",
				        tmsid, info[i][j].modification_time, modification_time);
				    fprintf(stderr,"   old range: %s", ctime(&info[i][j].min_start));
				    fprintf(stderr,"          to: %s", ctime(&info[i][j].max_start));
				    fprintf(stderr,"   new range: %s", ctime(&min_start));
				    fprintf(stderr,"          to: %s", ctime(&max_start));
			        }*/
			        info[i][j].modification_time = modification_time;
			        info[i][j].min_start = min_start;
			        info[i][j].max_start = max_start;
			        info[i][j].file_number = file;
			        info[i][j].start_offset = start_offset;
			        info[i][j].end_offset = end_offset;
			        info[i][j].line_count = line_count;
			    } /*else {
			        fprintf(stderr,"  Discarding outdated guide info for 0x%08x (%u > %u)\n",tmsid,
				    info[i][j].modification_time,modification_time);
			        fprintf(stderr,"   old range: %s", ctime(&info[i][j].min_start));
			        fprintf(stderr,"          to: %s", ctime(&info[i][j].max_start));
			        fprintf(stderr,"   new range: %s", ctime(&min_start));
			        fprintf(stderr,"          to: %s", ctime(&max_start));
			    }*/
			}
		    }
		}
	    }
        }
        fclose(fp);
	// Delete files more than 1 day old
	if(delete_old_files && (max_start != 0) && (max_start < (now-86400))) {
            fprintf(stderr,"   Deleting '%s'\n",filelist[file]);
	    unlink(filelist[file]);
	}
    }
    // Now go through the list of most-recent data and merge to one file
    fprintf(stdout,"stdin {\n");
    for(i=0; i<channelcount; i++) {
	if(channel_name[i] && channel_tuning[i])
		fprintf(stderr, "Copying channel %d %s\n", channel_tuning[i], channel_name[i]);
	for(j=0; (j<MAX_DAYS) && (info[i][j].modification_time != 0); j++) {
	    FILE *fp = fopen(filelist[info[i][j].file_number], "r");
	    if(fp == NULL) {
		fprintf(stderr, "ERROR: unable to open '%s'\n", filelist[info[i][j].file_number]);
	    } else {
	        fseek(fp, info[i][j].start_offset, SEEK_SET);
	        fprintf(stdout,"  Channel Lineup {\n");
		if(channel_name[i])
		    fprintf(stdout, "    Name                        %s\n",channel_name[i]);
		if(channel_tuning[i])
		    fprintf(stdout, "    Tuning                      %d\n",channel_tuning[i]);
	        for(k=0; k < info[i][j].line_count; k++) {
	  	    fgets(buffer,sizeof(buffer),fp);
		    fputs(buffer,stdout);
	        }
	        fclose(fp);
	    }
	}
    }
    fprintf(stdout,"}\n");
}
