/*==============================================================================
#  $Id: replaySchedule.c,v 116.40 2004/10/22 00:46:13 moyekj Exp $
# Copyright (C) 2005 Kevin J. Moye <moyekj@yahoo.com>
#
# Author: Kevin J. Moye
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
==============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#include "replaySchedule.h"
#include "hash.h"

// Parse an xml file for 6412 scheduled programs setup
// and return list of structures with replay channel information.
int parseXMLFile(
      char schedule_6412[], struct Channel ***channels, int *numC,
      char ***CategoryName, int *numCategories, int *cnum, int *catnum
    )
{
FILE *ifp;
char *s1, line[MAXLINE], s[MAXLINE], s2[MAXLINE];
int  i, j, stop=0;
struct Channel channel;

 if(!quiet) {
   fprintf(stderr,">> Parsing XML file '%s' ...\n", schedule_6412);
 }

 resetChannelStruct(&channel);

 ifp = fopen(schedule_6412, "r");
 if( ! ifp ) {
    fprintf(stderr, "*ERROR* Failed to open file: '%s'\n", schedule_6412);
    return(0);
 }

 // Read guide file and build list of all potential recordings
 while( fgets(line, MAXLINE, ifp) ) {
    i = strlen(line)-1;
    if( line[i] == '\n' ) {
       line[i] = '\0';
    }

    if ( !stop ) {
       // No more parsing needed beyond channels
       if ( !strcmp(line, "</channels>") ) {
          stop = 1;
       }

       // Category definition
       else if ( (s1 = strstr(line, "<category>")) ) {
          (*CategoryName)[(*numCategories)] = (char *)malloc(
             MAXLINE*sizeof(char)
          );
          sscanf(s1+strlen("<category>"), "%[^<]", s);
          strcpy((*CategoryName)[(*numCategories)], s);

          // Allocate more memory for categories if needed
          if ( ++(*numCategories) >= *catnum ) {
             (*catnum) += 100;
             (*CategoryName) = (char **)realloc(
                (*CategoryName), sizeof(char *)*(*catnum)
             );
          }
       }

       // Start of new channel definition
       else if ( (s1 = strstr(line, "<channel>")) ) {
          // Add previous channel hash to channel array
          if ( channel.Type && strlen(channel.Type) > 0 ) {
             // Allocate memory for and save this channel
             (*channels)[(*numC)] = (struct Channel *)malloc(
                sizeof(struct Channel)
             );
             memcpy((*channels)[(*numC)++], &channel, sizeof(struct Channel));
          }

          // Allocate more memory for channels if needed
          if ( *numC >= *cnum ) {
             (*cnum) += 100;
             (*channels) = (struct Channel **)realloc(
                (*channels), sizeof(struct Channel *)*(*cnum)
             );
          }
          // Reset channel struct
          resetChannelStruct(&channel);
       }

       // channel type
       else if ( (s1 = strstr(line, "<type>")) ) {
          sscanf(s1+strlen("<type>"), "%[^<]", s);
          strcpy(channel.Type, s);
       }

       // First run channel
       else if ( (s1 = strstr(line, "<firstRun>")) ) {
          sscanf(s1+strlen("<firstRun>"), "%[^<]", s);
          if( s[0] == '1' ) channel.firstRun = 1;
       }

       // Channel category name
       else if ( (s1 = strstr(line, "<category>")) ) {
          sscanf(s1+strlen("<category>"), "%[^<]", s);
          strcpy(channel.CategoryName, s);
       }

       // get theme keywords
       else if ( (s1 = strstr(line, "<keywords>")) ) {
          sscanf(s1+strlen("<keywords>"), "%[^<]", s);
          strcpy(channel.Keywords, s);
       }

       // get theme search keys
       else if ( (s1 = strstr(line, "<themeType>")) ) {
          sscanf(s1+strlen("<themeType>"), "%[^<]", s);
          strcpy(channel.SearchKeys, s);
       }

       // Title
       else if ( (s1 = strstr(line, "<title>")) ) {
          sscanf(s1+strlen("<title>"), "%[^<]", s);
          strcpy(channel.Title, s);
       }

       // tv channel name
       else if ( (s1 = strstr(line, "<channelName>")) ) {
          sscanf(s1+strlen("<channelName>"), "%[^<]", s);
          strcpy(channel.ChannelName, s);
       }

       // tv channel num
       else if ( (s1 = strstr(line, "<channelNum>")) ) {
          sscanf(s1+strlen("<channelNum>"), "%[^<]", s);
          channel.ChannelNum = atoi(s);
       }

       // start time
       // store in format: yyyyMMddhhmmss
       else if ( (s1 = strstr(line, "<start>")) ) {
          sscanf(s1+strlen("<start>"), "%[^<]", s);
          j = 0;
          for(i=0; i<strlen(s); ++i) {
             if( s[i] != '/' ) {
                s2[j++] = s[i];
             }
          }
          s2[j++] = '0';
          s2[j++] = '0';
          s2[j] = '\0';
          strcpy(channel.Time, s2);
       }

       // stop time => Calculate show length (from start time)
       else if ( (s1 = strstr(line, "<stop>")) ) {
          sscanf(s1+strlen("<stop>"), "%[^<]", s);
          j = 0;
          for(i=0; i<strlen(s); ++i) {
             if( s[i] != '/' ) {
                s2[j++] = s[i];
             }
          }
          s2[j++] = '0';
          s2[j++] = '0';
          s2[j] = '\0';
          if( strlen(channel.Time) > 0 ) {
             time_t start = guideTimeToUTC(channel.Time);
             time_t stop  = guideTimeToUTC(s2);
             channel.Minutes = (stop - start) / 60;
          }
       }

       // get days of week
       else if ( (s1 = strstr(line, "<days>")) ) {
          sscanf(s1+strlen("<days>"), "%[^<]", s);
          strcpy(channel.Days, s);
       }

       // pre padding
       else if ( (s1 = strstr(line, "<paddingBefore>")) ) {
          sscanf(s1+strlen("<paddingBefore>"), "%[^<]", s);
          channel.PaddingBefore = atoi(s);
       }

       // post padding
       else if ( (s1 = strstr(line, "<paddingAfter>")) ) {
          sscanf(s1+strlen("<paddingAfter>"), "%[^<]", s);
          channel.PaddingAfter = atoi(s);
       }

       // priority
       else if ( (s1 = strstr(line, "<priority>")) ) {
          sscanf(s1+strlen("<priority>"), "%[^<]", s);
          channel.PriorityBoost = atoi(s);
       }
    } // !stop
 } // while

 // Allocate memory for and save the last channel
 if ( channel.Type && (strlen(channel.Type) > 0) ) {
    (*channels)[(*numC)] = (struct Channel *)malloc(
       sizeof(struct Channel)
    );
    memcpy((*channels)[(*numC)++], &channel, sizeof(struct Channel));
 }

 fclose(ifp);

 if ( *numC == 0 ) {
    fprintf(stderr, "*ERROR* Missing/wrong channel info in file '%s'.\n",
       schedule_6412
    );
    return(0);
 }

 return(1);
}
