/*==============================================================================
# $Id: parsechannelguide.c,v 1.6 2003/09/26 23:17:23 pvanbaren Exp $
# Copyright (C) 2003 Philip Van Baren
#
# Author: Philip Van Baren
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
==============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <ctype.h>

#include "replaySchedule.h"

extern int DEBUG;

// match(buffer, word)
// Compare two strings, ignoring leading whitespace
// returns NULL if not equal
// returns pointer to the following data in first string if eqal
char *match(char *buffer, char *word) {
    int i,j;
    for(i=0; buffer[i] && isspace(buffer[i]); i++);
    for(j=0; buffer[i] && word[j] && (buffer[i]==word[j]); i++, j++);
    if(word[j])
	return NULL;
    // Skip to the beginning of the following word, or end of line
    for( ; buffer[i] && isspace(buffer[i]); i++);
    return buffer+i;
}

// skipword(buffer)
// skip over any whitespace-word-whitespace strings
// and return a pointer to the following data
// returns NULL if the end of line is reached
char *skipword(char *buffer) {
    int i;
    for(i=0; buffer[i] &&  isspace(buffer[i]); i++);
    for(   ; buffer[i] && !isspace(buffer[i]); i++);
    for(   ; buffer[i] &&  isspace(buffer[i]); i++);
    if(!buffer[i])
	return NULL;
    else
	return buffer+i;
}

#define MAX_LINE_LENGTH 1000
#define MAX_BLOCK_NEST 10
#define MAX_BLOCK_NAME_LENGTH 20

// Maximum number of channel entries we will handle
// Really should dynamically re-allocate, but I am lazy...
#define MAX_CHANNELS 2000
#define MAX_DAYS 20

int parseChannelGuide(
   char ChannelGuideScheduleFile[], struct Channel **channels, int numChan,
   struct Channel ***progs, int *pnum, int *numProg)
{
    int i,j,blocknest;
    char *ptr;
    char buffer[MAX_LINE_LENGTH];
    char blocklist[MAX_BLOCK_NEST][MAX_BLOCK_NAME_LENGTH];
    FILE *fp;
    struct Channel *chanmatch;
    struct prog program, progList[3];
    char **unique=NULL;
    int uniqueNum=0;
    int p=0, count=0;
    char channel_name[32];
    int channel_tuning=0;
    channel_name[0]=0;

    resetProgStruct(&program);
    for(i=0;i<3;i++)
        resetProgStruct(&progList[i]);

    blocknest = 0;

    fp = fopen(ChannelGuideScheduleFile, "r");
    if(fp == NULL) {
        fprintf(stderr,"ERROR: Could not open '%s'\n",ChannelGuideScheduleFile);
        return 0;
    }
    if(!quiet) {
       fprintf(stderr,"Scanning '%s' ...\n",ChannelGuideScheduleFile);
    }
    while(!feof(fp)) {
	fgets(buffer,sizeof(buffer),fp);
        // Strip trailing whitespace
        for(i=0; buffer[i]; i++);
        for(i--; (i>=0) && isspace(buffer[i]); buffer[i--]=0);
	if(strchr(buffer, '{') && (blocknest < MAX_BLOCK_NEST)) {
	    // Push the block type less leading and trailing spaces
	    for(i=0; buffer[i] && isspace(buffer[i]); i++);
	    for(j=0; buffer[i] && (buffer[i] != '{') && (j<MAX_BLOCK_NAME_LENGTH); i++,j++)
	        blocklist[blocknest][j] = buffer[i];
	    for(j-- ; (j>=0) && isspace(blocklist[blocknest][j]); j--)
	        blocklist[blocknest][j] = 0;
	    // Reset the program details when starting a Program
	    if(strcmp(blocklist[blocknest], "Program")==0) {
                // Reset program structure
                resetProgStruct(&program);
	    }
            // Reset the channel details when starting a Channel Lineup
            if(strcmp(blocklist[blocknest], "Channel Lineup")==0) {
                channel_tuning=0;
                channel_name[0]=0;
            }
	    //fprintf(stderr,"Start block: '%s'\n", blocklist[blocknest]);
	    blocknest++;
	} else if( (ptr = match(buffer, "TMS ID"))
		&& (ptr = skipword(ptr)) ) {
            program.tmsid = atol(ptr);
        } else if( (ptr = match(buffer, "Tuning")) ) {
	    channel_tuning = atol(ptr);
        } else if( (ptr = match(buffer, "Name")) ) {
	    strcpy(channel_name,ptr);
//fprintf(stderr,"TMSID=%u  Tuning=%d  Name='%s'\n",program.tmsid, channel_tuning, channel_name);
	} else if( (ptr = match(buffer, "Event Time")) ) {
	    program.ustart = atol(ptr);
	    if( (ptr = skipword(ptr)) ) {
		for(i=0,j=0; ptr[i]; i++) {
		    if(isdigit(ptr[i]))
			program.start[j++] = ptr[i];
                }
                program.start[j]=0;
                //fprintf(stderr,"Start: '%s' %u %u\n",program.start, program.ustart, guideTimeToUTC(program.start));
            }
            // Remember the earliest and latest start times
            // These are used for displaying single recordings within the
            // schedule times which don't have a matching program
            if( (program.ustart < minstarttime) || (minstarttime == 0) )
                minstarttime = program.ustart;
            if( (program.ustart > maxstarttime) || (maxstarttime == 0) )
                maxstarttime = program.ustart;
        } else if( (ptr = match(buffer, "Minutes"))
                    && (ptr = skipword(ptr)) ) {
            program.ustop = program.ustart + atol(ptr)*60;
	    program.stop[0] = 0;
        } else if( (ptr = match(buffer, "Title")) ) {
            strcpy(program.title, ptr);
        } else if( (ptr = match(buffer, "Episode")) ) {
            strcpy(program.subtitle, ptr);
        } else if( (ptr = match(buffer, "Description")) ) {
            strcpy(program.desc, ptr);
        } else if( (ptr = match(buffer, "Actor")) ) {
            strcpy(program.actors, ptr);
        } else if( (ptr = match(buffer, "Director")) ) {
            strcpy(program.directors, ptr);
        } else if( (ptr = match(buffer, "Flags")) ) {
            i = atol(ptr);
            if(i & 4) program.repeat = 1;
        } else if((blocknest > 0) && match(buffer, "}") ) {
	    // Pop the block type
	    //fprintf(stderr, "End block: '%s'\n", blocklist[blocknest-1]);
	    if((strcmp(blocklist[--blocknest],"Program") == 0) && (program.tmsid != 0)) {
                count++;
                // Copy in the channel name and number
                program.channelNum = channel_tuning;
                strcpy(program.channel, channel_name);
                if( count == 1 ) {
                    progList[1] = program;
                } else if( count == 2 ) {
                    progList[0] = progList[1];
                    progList[1] = program;
                } else if( count == 3 ) {
                    progList[2] = program;
                } else {
                    progList[0] = progList[1];
                    progList[1] = progList[2];
                    progList[2] = program;
		}
                p++;
                // Process when progList has at least 2 entries
                if ( count > 1 ) {
                   chanmatch = compareProgramToChannels(progList, channels, numChan);

                   if(chanmatch) {
                      /* Used for debugging matching for a specific show
                      if(DEBUG && (strcmp(chanmatch->title,"Mulheres Apaixonadas")==0)) {
                         if(chanmatch->AtOrAround)
                            fprintf(stderr, "AtOrAround match\n");
                         else
                            fprintf(stderr, "Direct match\n");
                      }
                      */

                       if( chanmatch->title[0] && ensureUniqueProgram(&unique, &uniqueNum, chanmatch) ) {
                           // Allocate more memory for progs if needed
                           if ( *numProg >= *pnum ) {
                               *pnum += 100;
                               (*progs) = (struct Channel **)realloc(
                                   (*progs), sizeof(struct Channel *)*(*pnum)
                               );
                           }
                           (*progs)[(*numProg)++] = chanmatch;
                       } else {
                           free(chanmatch);
                       }
                   }
               }
           }
        }
    }
    fclose(fp);

   // 1 more iteration on progList needed (to get last program)
   chanmatch = compareProgramToChannels(progList, channels, numChan);

   if ( chanmatch && strlen(chanmatch->title) > 0 ) {
      if ( ensureUniqueProgram(&unique, &uniqueNum, chanmatch) ) {
         if ( *numProg >= *pnum ) {
            (*pnum) += 100;
            (*progs) = (struct Channel **)realloc(
               (*progs), sizeof(struct Channel *)*(*pnum)
            );
         }
         (*progs)[(*numProg)++] = chanmatch;
      }
   }

   // Free up the allocated memory
   if(unique) {
      for(i=(uniqueNum-1); i>=0; i--) {
         if(unique[i]) free(unique[i]);
      }
      free(unique);
   }
   return(1);
}
