/*==============================================================================
#  $Id: replaySchedule.c,v 116.42 2005/05/20 20:27:47 moyekj Exp $
# Copyright (C) 2003 Kevin J. Moye <moyekj@yahoo.com>
#
# Author: Kevin J. Moye
# With contributions by Philip Van Baren
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Run this program with --help option for more info.
# Program to generate recording schedule for ReplayTV 4000/4500/5000/5500
# DVR series.
#
# NOTE: See the help (--help option) for info on channel mapping and many
#       other features.
==============================================================================*/

#define VERSION "$Id: replaySchedule.c,v 116.42 2005/05/20 20:27:47 moyekj Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#ifdef _WIN32
#  define S_ISREG(a) (((a)&_S_IFMT) == _S_IFREG)
#  define S_ISDIR(a) (((a)&_S_IFMT) == _S_IFDIR)
#else
#  include <unistd.h>
#endif

#include "replaySchedule.h"

static char _channel_[50];
static char _string_[MAXLINE];
struct chanMap **CHANMAP;
int    NUMMAP=0;

#include "guide.h"

int DEBUG=0;
int quiet=0;

// Globals used for flagging potentially missed programs
int flagMissing=0, NUM_MISSING=0;
struct Channel **MISSING=NULL;

time_t minstarttime = 0;
time_t maxstarttime = 0;

// Use global variables to remember the version info
// for XML output, because I'm too lazy to pass them
// the proper way
int replayversion=0;
int guideversion=0;
int RTV_OS = 0;

char *DayName[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
char *MonthName[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

int main(int argc, char *argv[])
{
   char replayIP[MAX_IP_NUM][256];
   char binFile[MAX_IP_NUM][256];
   char binFilePrefix[MAXLINE];
   char asciiGuideFile[MAX_IP_NUM][256];
   char asciiGuideFilePrefix[MAXLINE];
   char xmltvScheduleFile[MAX_SCHEDULE_FILES][256];
   char ChannelGuideFile[MAX_SCHEDULE_FILES][256];
   char htmlFile[256];
   char mapFile[256];
   char schedule_6412[256];
   char xmlOutput[256];
   int  BinDump=1, AsciiDump=1;
   struct Channel **channels = NULL;
   int numChan=0;
   struct Channel **progs = NULL, **sorted = NULL;
   int numCategory=0;
   char **CategoryName = NULL;
   int *CategoryIndex = NULL;
   int numProg=0, iarg;
   char TZ[20] = "";
   char string[25],*ptr;
   int i, pnum, xmltvScheduleFileCount=0, ChannelGuideFileCount=0, ipCount=0;
   struct stopTime *stopTimes = NULL;
   int stopCount=0;
   int noTheme=0;
   int dataDirect=0;
   int numSingleNoMatch=0;
   struct Channel **singleNoMatches = NULL;
   int numEmpty=0;
   struct Channel **emptyChannels = NULL;
   int cnum=100, catnum=100, indexalloc=100, indexes=0;
   struct Show **shows = NULL;
   int snum=100, numShows=0, todo=1;

   // Unset TZ env variable to prevent improper time computations
   // May need to comment this line out for Linux
#ifdef _WIN32
   putenv("TZ="); tzset();
#endif

   //testSearch();

   // Initialize options (override in command line)
   todo = 1;
   strcpy(mapFile,           "channelMap");
   strcpy(CONFIG.configFile, "replaySchedule.conf");
   CONFIG.ipnum = 0;
   CONFIG.recordedHtml[0] = '\0';
   strcpy(binFilePrefix,"");
   strcpy(asciiGuideFilePrefix,"");

   for(i=0; i<MAX_IP_NUM; ++i) {
      replayIP[i][0] = '\0';
      binFile[i][0] = '\0';
      asciiGuideFile[i][0] = '\0';
   }
   htmlFile[0] = '\0';
   xmlOutput[0] = '\0';
   schedule_6412[0] = '\0';

   // Process command-line options
   iarg = 1;
   while( iarg < argc ) {
      if (!strcmp(argv[iarg], "--help")) {
         useage(); 
         return(0);
      }
      if (!strcmp(argv[iarg], "--debug")) {
         DEBUG = 1;
      }
      if (!strcmp(argv[iarg], "--quiet")) {
         quiet = 1;
      }
      if (!strcmp(argv[iarg], "--b")) {
         BinDump = 0;
      }
      if (!strcmp(argv[iarg], "--a")) {
         AsciiDump = 0;
      }
      if (!strcmp(argv[iarg], "--notheme")) {
         noTheme = 1;
      }
      if (!strcmp(argv[iarg], "--dd")) {
         dataDirect = 1;
      }
      if (!strcmp(argv[iarg], "--notodo")) {
         todo = 0;
      }
      // Arguments that require an additional parameter
      // and therefore can't be the last argument in the list
      if( iarg < (argc-1) ) {
         if (!strcmp(argv[iarg], "--ip")) {
            strcpy(replayIP[ipCount++], argv[iarg+1]);
         }
         if (  (xmltvScheduleFileCount < MAX_SCHEDULE_FILES)
            && !strcmp(argv[iarg], "--schedule")) {
            strcpy(xmltvScheduleFile[xmltvScheduleFileCount++], argv[iarg+1]);
            // Split ',' separated names
            while( (xmltvScheduleFileCount < MAX_SCHEDULE_FILES)
                && (NULL != (ptr = strchr(xmltvScheduleFile[xmltvScheduleFileCount-1],',')) )) {
               *(ptr++) = 0;
               strcpy(xmltvScheduleFile[xmltvScheduleFileCount++], ptr);
            }
         }
         if (  (ChannelGuideFileCount < MAX_SCHEDULE_FILES)
            && !strcmp(argv[iarg], "--guide")) {
            strcpy(ChannelGuideFile[ChannelGuideFileCount++], argv[iarg+1]);
            // Split ',' separated names
            while( (ChannelGuideFileCount < MAX_SCHEDULE_FILES)
                && (NULL != (ptr = strchr(ChannelGuideFile[ChannelGuideFileCount-1],',')) )) {
               *(ptr++) = 0;
               strcpy(ChannelGuideFile[ChannelGuideFileCount++], ptr);
            }
         }
         
         if (!strcmp(argv[iarg], "--html")) {
            strcpy(htmlFile, argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--tz")) {
            strcpy(TZ, argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--xml")) {
            strcpy(xmlOutput, argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--config")) {
            strcpy(CONFIG.configFile, argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--missing")) {
            flagMissing = 1;
            strcpy(CONFIG.htmlMissingChannels,argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--bindump")) {
            BinDump = 1;
            strcpy(binFilePrefix,argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--asciidump")) {
            AsciiDump = 1;
            strcpy(asciiGuideFilePrefix,argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--recorded")) {
            strcpy(CONFIG.recordedHtml,argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--mapFile")) {
            strcpy(mapFile,argv[iarg+1]);
         }
         if (!strcmp(argv[iarg], "--6412")) {
            strcpy(schedule_6412,argv[iarg+1]);
         }
      }
      iarg++;
   }

   // Print out version info
   if(!quiet) fprintf(stderr,">> %s\n", VERSION);

   // Skipping themes message
   if(!quiet && noTheme) fprintf(stderr,">> Skipping theme channel listings in output\n");

   // Setup configuration (html settings)
   setupConfiguration();

   // If neither --b or --ip defined, display usage info and abort
   if(BinDump && replayIP[0][0] == '\0' && schedule_6412[0] == '\0') {
      useage();
      return(1);
   }

   // If no schedule files defined, use the default name
   if((xmltvScheduleFileCount==0) && (ChannelGuideFileCount==0)) {
      xmltvScheduleFileCount=1;
      strcpy(xmltvScheduleFile[0],"schedule.xml");
   }

   // Define .bin and .asc files
   for( i=0; i<ipCount; ++i ) {
      if( strlen(binFilePrefix) > 0 ) {
         sprintf(binFile[i], buildDumpFile(binFilePrefix,replayIP[i]));
      // Unless we force a prefix, use the direct-dump method
      //} else {
      //   sprintf(binFile[i],"%s.bin",replayIP[i]);
      }
      if( strlen(asciiGuideFilePrefix) > 0 ) {
         sprintf(asciiGuideFile[i],"%s.%s",asciiGuideFilePrefix,replayIP[i]);
         sprintf(asciiGuideFile[i],
            buildDumpFile(asciiGuideFilePrefix,replayIP[i])
         );
      // Unless we force a prefix, use the direct-dump method
      //} else {
      //   sprintf(asciiGuideFile[i],"%s.asc",replayIP[i]);
      }
   }

   // If no output defined, revert to HTML to sched.html
   if((xmlOutput[0] == 0) && (htmlFile[0] == 0))
      strcpy(htmlFile, "sched.html");

   // Set time zone if --tz option used
   // Eastern=>EST5EDT  Central=>CST6CDT  Pacific=>PST8PDT
   if( strlen(TZ) > 0 ) {
      sprintf(string, "TZ=%s", TZ);
      putenv(string); tzset();
      if(!quiet) fprintf(stderr,">> Set Timezone (TZ) to '%s'\n", TZ);
   }

   // Build channel map if mapFile exists
   if( fileIsFile(mapFile) ) {
      buildChannelMap(mapFile);
   }

   // Extract binary guide files
   if ( BinDump ) {
      for( i=0; i<ipCount; ++i ) {
         if( strlen(binFile[i]) > 0 )
            getReplayGuide(replayIP[i],binFile[i]);
      }
   }

   // Make ascii guide files from binary files
   if ( AsciiDump ) {
      for( i=0; i<ipCount; ++i ) {
         if( strlen(asciiGuideFile[i]) > 0 )
            makeAsciiGuide(binFile[i], asciiGuideFile[i]);
      }
   }

   // Extract replay channel info
   channels = (struct Channel **)malloc(sizeof(struct Channel *)*cnum);
   CategoryName = (char **)malloc(sizeof(char *)*catnum);
   CategoryIndex = (int *)malloc(sizeof(int)*indexalloc);
   if( CONFIG.recordedHtml[0] ) {
      shows = (struct Show **)malloc(sizeof(struct Show *)*snum);
   }

  if( schedule_6412[0] != '\0' ) {
    // XML channel definition parser
    i = parseXMLFile(
       schedule_6412, &channels, &numChan,
       &CategoryName, &numCategory, &cnum, &catnum
    );
    if( !i ) return(1);
  }
  else {
   for( i=0; i<ipCount; ++i ) {
      // Direct parser (no ascii guide dump file)
      if( strlen(asciiGuideFile[i]) == 0 ) {
          if ( ! convertReplayGuide(replayIP[i], &channels, &numChan,
             &CategoryIndex, &CategoryName, &numCategory,
             &cnum, &catnum, &indexes, &indexalloc,
             &shows, &numShows, &snum) ) {
#ifdef _DEBUG
	       {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	       }
#endif
             return(1);
          }

      // Ascii guide dump file parser
      } else if ( ! parseReplayGuide(replayIP[i], asciiGuideFile[i], &channels, &numChan,
             &CategoryIndex, &CategoryName, &numCategory,
             &cnum, &catnum, &indexes, &indexalloc,
             &shows, &numShows, &snum) ) {
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
         return(1);
      }
   }
  }

   if( CONFIG.recordedHtml[0] ) {
      if(!quiet) fprintf(stderr,">> Collected info on recorded shows.\n");
   }

   if (DEBUG) {
      // Print info on channels
      printf("** CHANNEL INFO **\n");
      printParsedInfo(channels, numChan);

      // Print info on recorded shows if specified
      if( CONFIG.recordedHtml[0] && numShows > 0 ) {
         printf("\n** RECORDED SHOW INFO **\n");
         for( i=0; i<numShows; ++i ) {
            printf("\n");
            printRecShowInfo(shows[i]);
         }
      }
   }

   // Extract recording schedule(s) (todo list) by parsing xml file
  if( todo ) {
   pnum = 100;
   progs = (struct Channel **)malloc(sizeof(struct Channel *)*pnum);

   // XMLTV file syntax
   if( dataDirect == 0 ) {
      for(i=0; i<xmltvScheduleFileCount; i++) {
         // Parse the xml file(s) to pull out missing programme stop times
         stopCount = computeMissingStopTimes(xmltvScheduleFile[i], &stopTimes);

         // Extract recordings
         if ( ! extractRecordingSchedule(
                  xmltvScheduleFile[i], channels, numChan,
                  &progs, &pnum, &numProg, stopTimes
                ) ) {
#ifdef _DEBUG
	      {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	      }
#endif
            return(1);
         }

         // Free allocated space for stopTimes
         if( stopTimes ) free(stopTimes);
      }
   }
   // DataDirect file syntax
   else {
      for(i=0; i<xmltvScheduleFileCount; i++) {
         // Extract recordings
         if ( ! extractRecordingScheduleDD(
                  xmltvScheduleFile[i], channels, numChan,
                  &progs, &pnum, &numProg, stopTimes
                ) ) {
#ifdef _DEBUG
	      {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	      }
#endif
            return(1);
         }
      }
   }

   // Extract recording schedule(s) by parsing channel guide files
   for(i=0; i<ChannelGuideFileCount; i++) {
      // Extract recordings
      if ( ! parseChannelGuide(
               ChannelGuideFile[i], channels, numChan,
               &progs, &pnum, &numProg
             ) ) {
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
         return(1);
      }
   }

   // Check for overlapping recordings
   checkForConflicts( &progs, numProg );

   // Add in single recordings which don't match the schedule
   // to detect and flag possible schedule time shifts
   checkForMissingSingleRecordings(
      channels, numChan, &singleNoMatches, &numSingleNoMatch
   );

   if( flagMissing ) {
      checkForEmptyRecurringChannels(
         channels, numChan, &emptyChannels, &numEmpty
      );
   }

   // Sort by start times chronologically
   sortProgs( &sorted, &progs, numProg );
   if (DEBUG) {
      printf("\n** TO DO LIST **\n");
      printShowInfo(sorted, numProg);
   }

   if(xmlOutput[0] != '\0') {
      // Create XML output
      createXmlSchedule(
         xmlOutput, replayIP[0], sorted, numProg,
         CategoryIndex, CategoryName, numCategory, noTheme
      );
   }
   if(htmlFile[0] != '\0') {
      // Create HTML schedule
      createHtmlSchedule(
         htmlFile, replayIP, ipCount, sorted, numProg, noTheme,
         singleNoMatches, numSingleNoMatch,
         emptyChannels, numEmpty
      );
   }
  } // end todo processing

   if( CONFIG.recordedHtml[0] ) {
      createRecordedSchedule(
         CONFIG.recordedHtml, replayIP, ipCount,
         shows, numShows, channels, numChan
      );
   }

   // Free up the allocated memory
   if(channels != NULL) {
	   for(i=(numChan-1); i>=0; i--)
		   free(channels[i]);
	   free(channels);
   }

   if(progs != NULL) {
	   for(i=(numProg-1); i>=0; i--)
		   free(progs[i]);
	   free(progs);
   }
   if(sorted != NULL) {
	   // Individual elements are the same pointers as progs
	   // so we don't need to free them again
	   free(sorted);
   }

   if(CategoryName != NULL) {
	for(i=(numCategory-1); i>=0; i--)
		free(CategoryName[i]);
	free(CategoryName);
   }
   if(CategoryIndex != NULL) {
	free(CategoryIndex);
   }

   if( NUMMAP > 0 ) {
      for(i=(NUMMAP-1); i>=0; i--) {
        if( CHANMAP[i] ) free(CHANMAP[i]);
      }
      if( CHANMAP ) free(CHANMAP);
   }

   if( NUM_MISSING > 0 ) {
      for(i=(NUM_MISSING-1); i>=0; i--) {
        if( MISSING[i] ) free(MISSING[i]);
      }
      if( MISSING ) free(MISSING);
   }

   if(shows != NULL) {
      for(i=(numShows-1); i>=0; i--)
         free(shows[i]);
      free(shows);
   }

   if(singleNoMatches) {
      for(i=(numSingleNoMatch-1); i>=0; i--)
         free(singleNoMatches[i]);
      free(singleNoMatches);
   }

#ifdef _DEBUG
   {
	   char buf[10];
	   fputs("Press <ENTER> to exit...",stderr);
	   fgets(buf,sizeof(buf),stdin);
   }
#endif
   return(0);
}

void useage() {
   printf("\n");
   printf("PROGRAM\n");
   printf("-------\n");
   printf("replaySchedule\n");
   printf("Version: %s\n", VERSION);
   printf("  Command line options:\n");
   printf("    --ip replayIP    (REQUIRED argument: RTV IP)\n");
   printf("                      NOTE: Use can use multiple --ip args\n");
   printf("                      to specify multiple RTVs\n");
   printf("    --schedule  file (xmltv file name - schedule.xml by default)\n");
   printf("                      NOTE: Use can use multiple --schedule args\n");
   printf("                      to specify multiple input schedules\n");
   printf("    --dd             (schedule file is in DataDirect format)\n");
   printf("    --html      file (output html schedule file)\n");
   printf("    --xml       file (output xml schedule file) for use by PRG\n");
   printf("    --recorded  file (output recorded shows to this html file)\n");
   printf("    --config    file (configuration control file to use:\n");
   printf("                      replaySchedule.config by default)\n");
   printf("    --notheme        (skip processing of theme-based channels)\n");
   printf("    --notodo         (don't generate html/xml schedule file)\n");
   printf("    --missing names  (Catch programs whose titles and channel\n");
   printf("                      names match those of a Recurring Channel\n");
   printf("                      but will not be recorded by the RTV.\n");
   printf("                      names=ALL         => flag for all channels\n");
   printf("                      names=CH1,CH2,... => flag only for CH1,CH2,...)\n");
   printf("                      names=!CH1,CH2,... => flag all except CH1,CH2,...)\n");
   printf("    --bindump   file (Use file.IP as binary dump file(s))\n");
   printf("    --asciidump file (Use file.IP as ascii dump file(s))\n");
   printf("    --mapFile   file (Use file as channel map file)\n");
   printf("    --help           (Display this help text and exit)\n");
   printf("    <<< These options for debug purposes >>>\n");
   printf("    --debug          (turn on debug mode for verbose output)\n");
   printf("    --b              (skip getting replay binary guide file)\n");
   printf("    --a              (skip building replay ascii guide file)\n");
   printf("    --tz string      (set a specific timezone)\n");
   printf("         Eastern=EST5EDT Central=CST6CDT Pacific=PST8PDT\n");
   printf("\n");
   printf("WINDOWS EXAMPLE\n");
   printf("---------------\n");
   printf("del schedule.xml\n");
   printf("xmltv.exe tv_grab_na --days 7 --listings schedule.xml\n");
   printf("replaySchedule --ip 192.168.1.10 --html bedroomToDo.htm\n");
   printf("\n");
   printf("CHANNEL MAPPING\n");
   printf("---------------\n");
   printf("In some cases the Replay channel names do not always match the\n");
   printf("channel names given in the xmltv schedule (for some channels).\n");
   printf("This can cause all those programs on these channels to not\n");
   printf("show up in the generated recording schedule html file.\n");
   printf("If this is the case you can supply a './channelMap' file to\n");
   printf("map xmltv channel names to Replay channel names. Syntax is:\n");
   printf("     xmltvChannelName replayChannelName\n");
   printf("Example:\n");
   printf("     USA USAP\n");
   printf("Maps xmltv channel name USA to Replay channel name USAP.\n");
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
   exit(1);
}

// Set configuration defaults.
// Override defaults if configuration file available.
int setupConfiguration() {
FILE *ifp;
char line[MAXLINE], **fields=NULL;
int  numF=0;
   // Setup defaults
   strcpy(CONFIG.htmlBackgroundColor,     "black");
   strcpy(CONFIG.htmlForegroundColor,     "white");
   strcpy(CONFIG.htmlConflictLoserColor,  "red");
   strcpy(CONFIG.htmlConflictLoserTitle,  "(Red => Conflict)");
   strcpy(CONFIG.htmlConflictWinnerColor, "yellow");
   strcpy(CONFIG.htmlConflictWinnerTitle, "(Yellow => Conflict Winner)");
   strcpy(CONFIG.htmlSingleNoMatchColor,  "orange");
   strcpy(CONFIG.htmlSingleNoMatchTitle,
      "Single Record Channels with no Match"
   );
   strcpy(CONFIG.htmlEmptyChannelColor,   "orange");
   strcpy(CONFIG.htmlEmptyChannelTitle,
      "Recurring Record Channels with no Matches"
   );
   strcpy(CONFIG.htmlMissingColor,        "beige");
   strcpy(CONFIG.htmlMissingTitle,        "RECORDINGS YOU MAY BE MISSING");
   CONFIG.htmlDontFlagMissingRepeats = 0;
   if( !flagMissing ) {
      strcpy(CONFIG.htmlMissingChannels,  "ALL");
   }
   strcpy(CONFIG.htmlMissingShowIgnore,   "");
   strcpy(CONFIG.htmlSingleChannelTag,    "[S]");
   strcpy(CONFIG.htmlRecurringChannelTag, "[R]");
   strcpy(CONFIG.htmlRecurringChannelAtOrAroundTag, "[RA]");
   strcpy(CONFIG.htmlThemeChannelTag,     "[T]");

   // Read config file and adjust options as needed
   if( fileIsFile(CONFIG.configFile) ) {
      ifp = fopen(CONFIG.configFile, "r");
      if( ! ifp ) {
         fprintf(stderr, "*WARNING* Failed to read config file: '%s'\n",
            CONFIG.configFile
         );
         return(0);
      }

      // Read configuration file one line at a time
      if( !quiet ) fprintf(stderr, ">> Reading config file: %s\n", CONFIG.configFile);
      while( fgets(line, MAXLINE, ifp) ) {
         line[strlen(line)-1] = '\0';
         // Skip comment lines
         if( line[0] != '#' ) {
            // Make an array of strings out of the current line
            splitLine(line, &fields, &numF);
            if( numF == 1 ) {
               if( strstr(fields[0], "htmlDontFlagMissingRepeats") ) {
                  CONFIG.htmlDontFlagMissingRepeats = 1;
               }
            }
            if( numF >= 2 ) {
               if( strstr(fields[0], "htmlBackgroundColor") ) {
                  strcpy(CONFIG.htmlBackgroundColor, fields[1]);
               }
               if( strstr(fields[0], "htmlForegroundColor") ) {
                  strcpy(CONFIG.htmlForegroundColor, fields[1]);
               }
               if( strstr(fields[0], "htmlConflictLoserColor") ) {
                  strcpy(CONFIG.htmlConflictLoserColor, fields[1]);
               }
               if( strstr(fields[0], "htmlConflictLoserTitle") ) {
                  strcpy(CONFIG.htmlConflictLoserTitle, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlConflictWinnerColor") ) {
                  strcpy(CONFIG.htmlConflictWinnerColor, fields[1]);
               }
               if( strstr(fields[0], "htmlConflictWinnerTitle") ) {
                  strcpy(CONFIG.htmlConflictWinnerTitle, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlSingleNoMatchColor") ) {
                  strcpy(CONFIG.htmlSingleNoMatchColor, fields[1]);
               }
               if( strstr(fields[0], "htmlSingleNoMatchTitle") ) {
                  strcpy(CONFIG.htmlSingleNoMatchTitle, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlMissingColor") ) {
                  strcpy(CONFIG.htmlMissingColor, fields[1]);
               }
               if( strstr(fields[0], "htmlMissingTitle") ) {
                  strcpy(CONFIG.htmlMissingTitle, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlEmptyChannelColor") ) {
                  strcpy(CONFIG.htmlEmptyChannelColor, fields[1]);
               }
               if( strstr(fields[0], "htmlEmptyChannelTitle") ) {
                  strcpy(CONFIG.htmlEmptyChannelTitle, joinFields(1,numF-1,fields));
               }
               if( !flagMissing && strstr(fields[0], "htmlMissingChannels") ) {
                  flagMissing = 1;
                  strcpy(CONFIG.htmlMissingChannels, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlMissingShowIgnore") ) {
                  strcpy(CONFIG.htmlMissingShowIgnore, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlSingleChannelTag") ) {
                  strcpy(CONFIG.htmlSingleChannelTag, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlRecurringChannelTag") ) {
                  strcpy(CONFIG.htmlRecurringChannelTag, joinFields(1,numF-1,fields)); 
               }
               if( strstr(fields[0], "htmlRecurringChannelAtOrAroundTag") ) {
                  strcpy(CONFIG.htmlRecurringChannelAtOrAroundTag, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "htmlThemeChannelTag") ) {
                  strcpy(CONFIG.htmlThemeChannelTag, joinFields(1,numF-1,fields));
               }
               if( strstr(fields[0], "replayMap") ) {
                  strcpy(CONFIG.ipmap[CONFIG.ipnum].ip, fields[1]);
                  strcpy(CONFIG.ipmap[CONFIG.ipnum].name,joinFields(2,numF-1,fields));
                  CONFIG.ipnum++;
               }
            }
            // Free up allocated memory
            freeLine(fields, numF);
         }
      }
      fclose(ifp);
   }
   return(1);
}

/*
 * Parse xmltv schedule file and pull out the missing stop times when possible.
 * Stores the resulting structures in stopTimes.
 */
int computeMissingStopTimes(char xmltvScheduleFile[], struct stopTime **stopTimes)
{
FILE *ifp;
char line[MAXLINE], channel[256], start[24];
char *s1;
int  i, num=0;
int numalloc=1000;

 if(!quiet) fprintf(stderr,">> Computing missing stop times in '%s'...\n", xmltvScheduleFile);
 ifp = fopen(xmltvScheduleFile, "r");
 if( ! ifp ) {
    fprintf(stderr, "*ERROR* Failed to open xml file: '%s'\n", xmltvScheduleFile);
    exit(1);
 }

 (*stopTimes) = (struct stopTime *)malloc(sizeof(struct stopTime)*numalloc);

 // Read guide file and build list of all potential recordings
 while( fgets(line, MAXLINE, ifp) ) {
    i = strlen(line)-1;
    if( line[i] == '\n' ) {
       line[i] = '\0';
    }

    // Parse the xml file, 1 line at a time

    if( hasStr(line, "<programme start",1) ) {
       // Pull out channel name
       s1 = strstr(line, "channel=");
       sscanf(s1+9,"%[^\"]",channel);

       // programme line without stop=
       if( ! hasStr(line, "stop=", 1) ) {
          // Make room in memory for another stopTime struct
		   if(num >= numalloc) {
			   numalloc += 1000;
               (*stopTimes) = (struct stopTime *)realloc((*stopTimes), sizeof(struct stopTime)*(numalloc));
		   }
          // Add to array of programmes we need stop time for
          strcpy((*stopTimes)[num].channel, channel);
          strcpy((*stopTimes)[num++].stop, "");
       }

       else {
          // See if this matches a channel name we are looking for a stop time
          for( i=0; i<num; ++i ) {
             if( !strcmp(channel, (*stopTimes)[i].channel) && strlen((*stopTimes)[i].stop) == 0 ) {
                // Grab start time to use as stop time for last program
                if( (s1 = strstr(line, "start=")) ) {
                   sscanf(s1+7, "%[^ ]", start);
                   // Set stop time to this grabbed start time
                   strcpy((*stopTimes)[i].stop, start);
                }
             }
          }
       }
    }
 }
 fclose(ifp);
 return num;
}

// Parse the ascii replay guide file extracted from ReplayPC
// and return list of structures with replay channel information.
int parseReplayGuide(
      char replayIP[], char asciiGuideFile[], struct Channel ***channels, int *numC,
      int **CategoryIndex, char ***CategoryName, int *numCategories,
      int *cnum, int *catnum, int *indexes, int *indexalloc,
      struct Show ***shows, int *numS, int *snum
    )
{
FILE *ifp;
char line[MAXLINE], block[MAXLINE];
char s1[MAXLINE], **fields=NULL;
int  i, j, count=0, numF=0;
int  stop=0;
struct Channel channel;
struct Show show;

 if(!quiet) {
   fprintf(stderr,">> Parsing Replay channel information for '%s' ...\n",
      replayIP
   );
 }

 resetChannelStruct(&channel);
 strcpy(channel.ip, replayIP);

 resetShowStruct(&show);
 strcpy(show.ip, replayIP);

 ifp = fopen(asciiGuideFile, "r");
 if( ! ifp ) {
    fprintf(stderr, "*ERROR* Failed to open file: '%s'\n", asciiGuideFile);
    return(0);
 }

 // Read guide file and build list of all potential recordings
 while( fgets(line, MAXLINE, ifp) ) {
    i = strlen(line)-1;
    if( line[i] == '\n' ) {
       line[i] = '\0';
    }

    if ( !stop ) {

      // Make an array of strings out of the current line
      splitLine(line, &fields, &numF);
      if(numF <= 0) {
         stop = stop;
      } else {
         // Keep track of block name we are in
         if( !strcmp(fields[numF-1],"{") ) {
            strcpy(block, "");
            count = 0;
            for( i=0; i<numF-1; ++i ) {
               strcat(block, fields[i]);
               count += strlen(fields[i]);
            }
            block[count+1] = '\0';
         }

         // No more parsing when recorded shows reached
         if ( !strcmp(block, "ReplayShows") ) {
            stop = 1;
         }

         // Replay and Guide version numbers
         else if ( !strcmp(fields[0], "Version") && (numF == 3) ) {
            replayversion = atoi(fields[2]);
            // guideversion is the major portion of the software version number
            // where guideversion==1 is OS==4, guideversion==2 is OS==5
            guideversion = replayversion & 0xFFFF;
            // Replayversion is the dotted portion of the software version number
            replayversion >>= 16;
            RTV_OS = 3 + guideversion;
            if(!quiet) fprintf(stderr,"   (%d.%d software version)\n", RTV_OS, replayversion);
         }

         // Category index
         else if ( hasStr(line, "Category Index", 1) ) {
            (*CategoryIndex)[(*indexes)++] = atoi(fields[numF-1]);
            // Allocate more memory for categories if needed
            if ( *indexes >= *indexalloc ) {
               (*indexalloc) += 100;
               (*CategoryIndex) = (int *)realloc(
                  (*CategoryIndex), sizeof(int)*(*indexalloc)
               );
            }
         }

         // Category definition
         else if ( hasStr(line, "Category Name", 1) ) {
            (*CategoryName)[(*numCategories)] = (char *)malloc(MAXLINE*sizeof(char));
            strcpy((*CategoryName)[(*numCategories)], fields[3]);
            for (j=4; j<numF; j++) {
               strcat((*CategoryName)[(*numCategories)], " ");
               strcat((*CategoryName)[(*numCategories)], fields[j]);
            }

            // Allocate more memory for categories if needed
            if ( ++(*numCategories) >= *catnum ) {
               (*catnum) += 100;
               (*CategoryName) = (char **)realloc(
                  (*CategoryName), sizeof(char *)*(*catnum)
               );
            }
         }

         // Start of new replay channel definition
         else if ( hasStr(line, "Guide Channel {", 1) ) {
            strcpy(channel.ip, replayIP);
            // Add previous channel hash to channel array
            if ( channel.Type && strlen(channel.Type) > 0 ) {
               // Allocate memory for and save this channel
               (*channels)[(*numC)] = (struct Channel *)malloc(
                  sizeof(struct Channel)
               );
               memcpy((*channels)[(*numC)++], &channel, sizeof(struct Channel));
            }

            // Allocate more memory for channels if needed
            if ( *numC >= *cnum ) {
               (*cnum) += 100;
               (*channels) = (struct Channel **)realloc(
                  (*channels), sizeof(struct Channel *)*(*cnum)
               );
            }
            // Reset channel struct
            resetChannelStruct(&channel);
            strcpy(channel.ip, replayIP);
         }

         // replay channel type
         else if ( hasStr(line, "Guide Channel Type", 1) ) {
            strcpy(channel.Type, fields[numF-1]);
         }

         // First run channel (5.0 software)
         else if ( !strcmp(fields[numF-1], "FirstRun") ) {
            channel.firstRun = 1;
         }

         // Channel category name
         else if ( !strcmp(block,"GuideChannel") && !strcmp(fields[0], "category") ) {
            if( numF == 2 ) {
               strcpy(channel.CategoryName, "Uncategorized");
            } else {
               strcpy(channel.CategoryName, fields[2]);
               for(i=3; i<numF; ++i) {
                  strcat(channel.CategoryName, " ");
                  strcat(channel.CategoryName, fields[i]);
               }
            }
         }

         // Channel ID
         else if ( !strcmp(block,"GuideChannel") && hasStr(line, "Channel ID", 1) ) {
            channel.channelID = atoi(fields[2]);
         }

         // replay channel name
         else if ( !strcmp(fields[0], "label") ) {
            strcpy(s1, joinFields(1, numF-1, fields) );
            // Weird character replaces ' in channel guide dump
            fixWeirdChars(s1);
            strcpy(channel.Name, s1);
         }

         // get scheduled minutes
         else if ( !strcmp(fields[0], "Minutes") ) {
            if ( numF > 1 ) {
               if( strcmp(fields[numF-1],"0") ) {
                  channel.Minutes = atoi(fields[numF-1]);
               }
            }
         }

         // get theme keywords
         else if ( !strcmp(block,"ReplayTheme") && numF > 1 && hasStr(fields[0], "Keyw", 1) ) {
            strcpy(s1, joinFields(1, numF-1, fields));
            // Weird character replaces ' in channel guide dump
            fixWeirdChars(s1);
            strcpy(channel.Keywords, s1);
         }

         // get theme search keys
         else if ( !strcmp(block,"ReplayTheme") && numF > 2 && !strcmp(fields[0],"Flags") ) {
            strcpy(channel.SearchKeys, joinFields(2, numF-1, fields));
         }

         // get Title if there is one
         else if ( !strcmp(fields[0],"Title") ) {
            strcpy(s1, joinFields(1, numF-1, fields));
            // Weird character replaces ' in channel guide dump
            fixWeirdChars(s1);
            strcpy(channel.Title, s1);
         }

         // get Show Flags if there is one
         else if ( !strcmp(block,"ReplayShow") && !strcmp(fields[0],"Flags") ) {
            strcpy(channel.ShowFlags, joinFields(1, numF-1, fields));
         }

         // get the tv channel name if there is one
         else if ( !strcmp(block,"HeadendChannel") && !strcmp(fields[0],"Tuning") ) {
            channel.ChannelNum = atoi(fields[numF-1]);
         }
         else if ( !strcmp(block,"HeadendChannel") && !strcmp(fields[0],"Name") ) {
            strcpy(channel.ChannelName, joinFields(1, numF-1, fields));
         }
         else if ( !strcmp(block,"HeadendChannel") && !strcmp(fields[0],"TMS")) {
            channel.tmsid = atol(fields[numF-1]);
         }
         // get start time if there is one
         // store in format: yyyyMMddhhmmss
         else if ( !strcmp(block,"Program") && hasStr(line,"Event Time",1) ) {
            strcpy(channel.Time, parseTimeToGuideTime(joinFields(3, numF-1, fields)));
         }

         // get days_of_week
         else if ( !strcmp(fields[0],"days_of_week") ) {
            strcpy(channel.Days, joinFields(2, numF-1, fields));
         }

         // determine if guaranteed
         else if ( !strcmp(fields[0],"Guaranteed") ) {
            channel.Guaranteed = atoi(fields[numF-1]);
            if( channel.Guaranteed ) channel.Guaranteed = 1;
         }

         // get channel creation date
         else if ( hasStr(line,"Channel ID",1) ) {
            strcpy(channel.ChannelDate,
               parseTimeToGuideTime(joinFields(3, numF-1, fields))
            );
         }

         // get version 5.x priority override
         else if ( hasStr(line,"priority",1) ) {
            channel.PriorityBoost = atoi(fields[numF-1]);
         }

         // pre and post padding
         else if ( hasStr(line,"Padding Before",1) ) {
            channel.PaddingBefore = atoi(fields[numF-1]);
         }
         else if ( hasStr(line,"Padding After",1) ) {
            channel.PaddingAfter = atoi(fields[numF-1]);
         }
      }

    } // !stop

    // Parse Replay Shows (already recorded programs) if specified
    if( stop && CONFIG.recordedHtml[0] ) {
      // Make an array of strings out of the current line
      splitLine(line, &fields, &numF);
      if(numF > 0) {
         // Keep track of block name we are in
         if( !strcmp(fields[numF-1],"{") && strcmp(fields[0],"Movie") && strcmp(fields[0],"Parts") ) {
            strcpy(block, "");
            count = 0;
            for( i=0; i<numF-1; ++i ) {
               strcat(block, fields[i]);
               count += strlen(fields[i]);
            }
            block[count+1] = '\0';
         }

         // Channel ID
         if( hasStr(line,"Channel ID",1) ) {
            show.channelID = atoi(fields[2]);
         }

         // Headend Channel block
         if ( !strcmp(block,"HeadendChannel") ) {
            // Channel number
            if( !strcmp(fields[0],"Tuning") ) {
               show.ChannelNum = atoi(fields[numF-1]);
            }
            // Channel name
            else if ( !strcmp(fields[0],"Name") ) {
               strcpy(show.ChannelName, fields[numF-1]);
            }
         }

         // Program block
         else if ( !strcmp(block,"Program") ) {
            // Other Info
            if ( !strcmp(fields[0],"Flags") ) {
               strcpy(s1, parseTimeToGuideTime(joinFields(2, numF-1, fields)));
               fixWeirdChars(s1);
               strcpy(show.OtherInfo, s1);
            }
            // Time
            else if ( hasStr(line,"Event Time",1) ) {
               strcpy(s1, parseTimeToGuideTime(joinFields(3, numF-1, fields)));
               show.ustart = guideTimeToUTC(s1);
            }
            // Minutes
            else if ( !strcmp(fields[0],"Minutes") ) {
               show.Minutes = atoi(fields[numF-1]);
               show.ustop = show.ustart + 60*show.Minutes;
            }
            // Title
            else if ( !strcmp(fields[0],"Title") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Title, s1);
            }
            // Episode
            else if ( !strcmp(fields[0],"Episode") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Episode, s1);
            }
            // Description
            else if ( !strcmp(fields[0],"Description") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Description, s1);
            }
            // Actors
            else if ( !strcmp(fields[0],"Actor") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Actors, s1);
            }
            // Guests
            else if ( !strcmp(fields[0],"Guest") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Guests, s1);
            }
            // Producers
            else if ( !strcmp(fields[0],"Producer") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Producers, s1);
            }
            // Directors
            else if ( !strcmp(fields[0],"Director") ) {
               strcpy(s1, joinFields(1, numF-1, fields));
               fixWeirdChars(s1);
               strcpy(show.Directors, s1);
            }
         }

         // Start of new replay show definition
         if ( hasStr(line, "Replay Show {", 1) ) {
            strcpy(show.ip, replayIP);
            // Add previous show hash to show array
            if ( show.Title && strlen(show.Title) > 0 ) {
               // Allocate memory for and save this show
               (*shows)[(*numS)] = (struct Show *)malloc(
                  sizeof(struct Show)
               );
               memcpy((*shows)[(*numS)++], &show, sizeof(struct Show));
            }

            // Allocate more memory for shows if needed
            if ( *numS >= *snum ) {
               (*snum) += 100;
               (*shows) = (struct Show **)realloc(
                  (*shows), sizeof(struct Show *)*(*snum)
               );
            }
            // Reset show struct
            resetShowStruct(&show);
            strcpy(show.ip, replayIP);
         }
       }
    }

 } // while

 // Allocate memory for and save the last channel
 if ( channel.Type && (strlen(channel.Type) > 0) ) {
    (*channels)[(*numC)] = (struct Channel *)malloc(
       sizeof(struct Channel)
    );
    memcpy((*channels)[(*numC)++], &channel, sizeof(struct Channel));
 }

 // Allocate memory for and save the last show if relevant
 if( CONFIG.recordedHtml[0] ) {
    if ( show.Title && (strlen(show.Title) > 0) ) {
       (*shows)[(*numS)] = (struct Show *)malloc(
          sizeof(struct Show)
       );
       memcpy((*shows)[(*numS)++], &show, sizeof(struct Show));
    }
 }

 fclose(ifp);

 // free up the allocated memory
 freeLine(fields, numF);

 if ( *indexes != *numCategories ) {
    fprintf(stderr, "*ERROR* Missing/wrong category info in file '%s'.\n",
       asciiGuideFile
    );
    return 0;
 }
 if ( *numC == 0 ) {
    fprintf(stderr, "*ERROR* Missing/wrong channel info in file '%s'.\n",
       asciiGuideFile
    );
    return(0);
 }

 return(1);
}

// New XMLTV (>=0.5.11) made it a little harder to determine channel info
// so that's what this struct and function are for
struct CHMAP {
   char name[512];
   char value[256];
   int  num;
};
struct CHMAP chMap[MAXCHAN];

struct CHMAP findChan(int numChan, char *channel) {
   int i;
   for( i=0; i<numChan; i++ ) {
      if( !strcmp(chMap[i].name, channel) ) {
         return(chMap[i]);
      }
   }
   fprintf(stderr, "*ERROR* Cannot find channel: %s\n", channel);
   exit(1);
}

// Look for single recordings in the time span
// which do not have matching items in the schedule
int checkForMissingSingleRecordings(
   struct Channel **channels, int numChan, 
   struct Channel ***singleNoMatches, int *numSingleNoMatch )
{
   int i;
   time_t utc;
   
   if(!quiet) fprintf(stderr,">> Checking for missing Single recordings ...\n");

   for(i=0; i<numChan; i++) {
      if( ! strcmp(channels[i]->Type,"Single") 
         && (channels[i]->matchcount == 0) ) {
         utc = guideTimeToUTC(channels[i]->Time);
         if( (utc >= minstarttime) && (utc <= maxstarttime) ) {
            if ( *numSingleNoMatch == 0 ) {
               (*singleNoMatches) = (struct Channel **)malloc(
                  sizeof(struct Channel *)
               );
            } else {
               (*singleNoMatches) = (struct Channel **)realloc(
                  (*singleNoMatches),
                  sizeof(struct Channel *)*((*numSingleNoMatch)+1)
               );
            }
            (*singleNoMatches)[*numSingleNoMatch] = (struct Channel *)malloc(
               sizeof(struct Channel)
            );
            resetChannelStruct((*singleNoMatches)[*numSingleNoMatch]);
            memcpy((*singleNoMatches)[*numSingleNoMatch],channels[i],sizeof(struct Channel));
            (*numSingleNoMatch)++;
         }
      }
   }
   return(1);
}

// Look for recurring recording channels with no
// matches in the schedule.
// This is to catch recurring channel titles that don't
// match show titles because the show title has changed.
// This also helps catch recurring channels for series
// that are no longer airing so that one can cleanup.
int checkForEmptyRecurringChannels(
   struct Channel **channels, int numChan, 
   struct Channel ***emptyChannels, int *numEmpty )
{
   int i;
   
   if(!quiet) fprintf(stderr,">> Checking for empty Recurring channels ...\n");
   for(i=0; i<numChan; i++) {
      if( ! strcmp(channels[i]->Type,"Recurring") 
         && (channels[i]->matchcount == 0) ) {
         if ( *numEmpty == 0 ) {
            (*emptyChannels) = (struct Channel **)malloc(
               sizeof(struct Channel *)
            );
         } else {
            (*emptyChannels) = (struct Channel **)realloc(
               (*emptyChannels),
               sizeof(struct Channel *)*((*numEmpty)+1)
            );
         }
         (*emptyChannels)[*numEmpty] = (struct Channel *)malloc(
            sizeof(struct Channel)
         );
         resetChannelStruct((*emptyChannels)[*numEmpty]);
         memcpy((*emptyChannels)[*numEmpty],channels[i],sizeof(struct Channel));
         (*numEmpty)++;
      }
   }
   return(1);
}

// Extract recording schedule
int extractRecordingSchedule(
   char xmltvScheduleFile[], struct Channel **channels, int numChan,
   struct Channel ***progs, int *pnum, int *numProg,
   struct stopTime *stopTimes ) {

   FILE *ifp;
   char	line[MAXLINE], channel[50], s[MAXLINE];
   char *s1, **unique=NULL;
   int  i, p=0, count=0;
   int  uniqueNum=0;

   struct Channel *match;
   struct prog program, progList[3];
   struct CHMAP chmp;
   int chan=0;
   int stopCount=0;
   int isxmltv=0;

   resetProgStruct(&program);
   for(i=0;i<3;i++)
	   resetProgStruct(&progList[i]);

   if(!quiet) fprintf(stderr,">> Parsing '%s' to determine schedule ...\n", xmltvScheduleFile);

   ifp = fopen(xmltvScheduleFile, "r");
   if( ! ifp ) {
      fprintf(stderr, "*ERROR* Failed to open file: '%s'\n", xmltvScheduleFile);
      return(0);
   }

   while( fgets(line, MAXLINE, ifp) ) {
      i = strlen(line)-1;
      if( line[i] == '\n' ) {
         line[i] = '\0';
      }

      // Parse the xml file, 1 line at a time
      if( hasStr(line, "DOCTYPE",1) && hasStr(line, "xmltv",1) ) {
         isxmltv = 1;
      }

      // channel id => XMLTV >= 0.5.11 => add to channel map
      if( hasStr(line, "<channel id",1) ) {
         s1 = strstr(line, "<channel id=\"");
         sscanf(s1+13, "%[^\"]", s);
         strcpy(chMap[chan].name, s);
      }

      // <display-name> => XMLTV >= 0.5.11 => contains real channel name
      if( hasStr(line, "<display-name>",1) ) {
         s1 = strstr(line, "<display-name>");
         if( hasStr(s1, " ",1) ) {
            if( hasStr(s1, "Channel ", 1) ) {
               sscanf(s1+22, "%d %s", &chMap[chan].num, channel);
               sscanf(channel, "%[^<]", s);
               strcpy(chMap[chan++].value, s);
            }
            else {
               sscanf(s1+14, "%d %s", &chMap[chan].num, channel);
               sscanf(channel, "%[^<]", s);
               strcpy(chMap[chan++].value, s);
            }
         }
      }

      // New program definition
      if( hasStr(line, "<programme",1) ) {
         // Reset program structure
         resetProgStruct(&program);

         // Start and stop times
         s1 = strstr(line, "start=\"");
         strncpy(program.start, s1+7, 14); program.start[14] = '\0';
         program.ustart = guideTimeToUTC(program.start);

         s1 = strstr(line, "stop=\"");
         if( s1 ) {
            strncpy(program.stop, s1+6, 14); program.stop[14] = '\0';
            program.ustop = guideTimeToUTC(program.stop);
         }
         else {
            // XMLTV didn't give stop time, so grab from stopTimes array
            if( strlen(stopTimes[stopCount].stop) != 0 ) {
               strcpy(program.stop, stopTimes[stopCount].stop);
               program.ustop = guideTimeToUTC(program.stop);
            }
            stopCount++;
         }

         // Remember the earliest and latest start times
         // These are used for displaying single recordings within the
         // schedule times which don't have a matching program
         if( (program.ustart < minstarttime) || (minstarttime == 0) )
            minstarttime = program.ustart;
         if( (program.ustart > maxstarttime) || (maxstarttime == 0) )
            maxstarttime = program.ustart;

         // Channel number and name
         s1 = strstr(line, "channel=\"");
         if( chan > 0 ) {
            // New XMLTV format requires finding channel name
            sscanf(s1+9, "%[^\"]", channel);
            chmp = findChan(chan, channel);
            strcpy(channel, chmp.value);
            program.channelNum = chmp.num;
            program.tmsid = 0; 
         } else {
            // Old XMLTV format has channel name directly in programme
            sscanf(s1+9, "%d %[^\"]", &program.channelNum, channel);
         }
         strcpy(program.channel, channelMap(channel));
      }

      // Title
      else if( (s1 = strstr(line, "<title>")) ) {
         sscanf(s1+7, "%[^<]", s);
         strcpy(program.title, fixAmpersans(s));
      }
      else if( (s1 = strstr(line, "<title lang=")) ) {
         s1 = strstr(line, ">");
         sscanf(s1+1, "%[^<]", s);
         strcpy(program.title, fixAmpersans(s));
      }

      // Sub-title
      else if( (s1 = strstr(line, "<sub-title>")) ) {
         sscanf(s1+11, "%[^<]", s);
         strcpy(program.subtitle, fixAmpersans(s));
      }
      else if( (s1 = strstr(line, "<sub-title lang=")) ) {
         s1 = strstr(line, ">");
         sscanf(s1+1, "%[^<]", s);
         strcpy(program.subtitle, fixAmpersans(s));
      }

      // Description
      else if( (s1 = strstr(line, "<desc>")) ) {
         sscanf(s1+6, "%[^<]", s);
         strcpy(program.desc, fixAmpersans(s));
      }
      else if( (s1 = strstr(line, "<desc lang=")) ) {
         s1 = strstr(line, ">");
         sscanf(s1+1, "%[^<]", s);
         strcpy(program.desc, fixAmpersans(s));
      }

      // Directors (tv_imdb listings)
      else if( (s1 = strstr(line, "<director>")) ) {
         sscanf(s1+10, "%[^<]", s);
         strcat(program.directors, " ");
         strcat(program.directors, s);
      }

      // Actors (tv_imdb listings)
      else if( (s1 = strstr(line, "<actor>")) ) {
         sscanf(s1+7, "%[^<]", s);
         strcat(program.actors, " ");
         strcat(program.actors, s);
      }

      // Repeat
      else if( strstr(line, "<previously-shown") ) {
         program.repeat = 1;
      }

      // End of program definition => time to process
      else if ( hasStr(line, "</programme>", 1) ) {
         count++;
         if( count == 1 ) {
            progList[1] = program;
         }
         else if( count == 2 ) {
            progList[0] = progList[1];
            progList[1] = program;
         }
         else {
            progList[0] = progList[1];
            progList[1] = progList[2];
            progList[2] = program;
         }
         p++;

         // Process when progList has at least 2 entries
         if ( count > 1 ) {
            match = compareProgramToChannels(progList, channels, numChan);

            if(match) {
               if( match->title[0] && ensureUniqueProgram(&unique, &uniqueNum, match) ) {
                  // Allocate more memory for progs if needed
                  if ( *numProg >= *pnum ) {
                     *pnum += 100;
                     (*progs) = (struct Channel **)realloc(
                        (*progs), sizeof(struct Channel *)*(*pnum)
                     );
                  }
                  (*progs)[(*numProg)++] = match;
               } else {
                  free(match);
               }
            }
         }
      }
   }

   fclose(ifp);

   // 1 more iteration on progList needed (to get last program)
   match = compareProgramToChannels(progList, channels, numChan);

   if ( match && strlen(match->title) > 0 ) {
      if ( ensureUniqueProgram(&unique, &uniqueNum, match) ) {
         if ( *numProg >= *pnum ) {
            (*pnum) += 100;
            (*progs) = (struct Channel **)realloc(
               (*progs), sizeof(struct Channel *)*(*pnum)
            );
         }
         (*progs)[(*numProg)++] = match;
      }
   }

   if( ! isxmltv ) {
      fprintf(stderr, "*ERROR* %s not in xmltv format... aborting\n",
         xmltvScheduleFile
      );
      exit(1);
   }

   // Free up the allocated memory
   if(unique) {
      for(i=(uniqueNum-1); i>=0; i--) {
         if(unique[i]) free(unique[i]);
      }
      free(unique);
   }
   return(1);
}

void buildChannelMap(char mapFile[]) {
   FILE *ifp;
   char **fields;
   char line[MAXLINE];
   int  i, numF;
   int numalloc=500;

   ifp = fopen(mapFile, "r");
   if (!ifp) {
      fprintf(stderr, "Couldn't open %s\n", mapFile);
      return;
   }

   if(!quiet) printf("--- NOTE: Mapping xml -> replay channel names\n");
   CHANMAP = (struct chanMap **)malloc(sizeof(struct chanMap *)*numalloc);
   while( fgets(line, MAXLINE, ifp) ) {
      i = strlen(line)-1;
      if( line[i] == '\n' ) {
         line[i] = '\0';
      }

      // Make an array of strings out of the current line
      splitLine(line, &fields, &numF);
      if(numF == 2) {
         if( NUMMAP >= numalloc) {
			 numalloc += 100;
             CHANMAP = (struct chanMap **)realloc(
               CHANMAP, (numalloc)*sizeof(struct chanMap *)
             );
         }
         CHANMAP[NUMMAP] = (struct chanMap *)malloc(sizeof(struct chanMap));
         strcpy(CHANMAP[NUMMAP]->name, fields[0]);
         strcpy(CHANMAP[NUMMAP]->mapped, fields[1]);
         if(!quiet) printf("--- %s -> %s\n", CHANMAP[NUMMAP]->name, CHANMAP[NUMMAP]->mapped);
         NUMMAP++;
      }
	  // free up the allocated memory
	  freeLine(fields, numF);
   }

   fclose(ifp);
}

char *channelMap(char *channel) {
   int i;

   strcpy(_channel_, channel);
   for(i=0; i<NUMMAP; ++i) {
      if( !strcmp(channel, CHANMAP[i]->name) ) {
         strcpy(_channel_,CHANMAP[i]->mapped);
         return(_channel_);
      }
   }
   return(_channel_);
}

// Check matched program against list of already matched programs
// Unique => 1, otherwise return 0
int ensureUniqueProgram( char ***unique, int *uniqueNum, struct Channel *match ) {
char e[MAXLINE];
int  isRecurring=0, isUnique=1;
int  i;
struct Time matchTime;

   if( !strcmp(match->Type, "Recurring") ) {
      isRecurring = 1;
   }

   if( isRecurring ) {
      // NOTE: Recurring channels: Only 1 match allowed per day
      matchTime = guideTimeToLocalTime(match->start);
      sprintf(e, "%s %d %s %s %d",
         match->ip, match->channelNum, match->title, match->ChannelDate, matchTime.mday
      );
   } else {
      sprintf(e, "%s %d %s %s", match->ip, match->channelNum, match->title, match->start);
   }
   for( i=0; i<(*uniqueNum); ++i ) {
      if( !strcmp((*unique)[i],e) ) {
         // Identical program => not unique
         isUnique = 0;
      }
   }

   if( isUnique ) {
      // Allocate more memory to store this new program in unique list
      if( *uniqueNum == 0 ) {
         (*unique) = (char **)malloc(sizeof(char *));
      } else {
         (*unique) = (char **)realloc((*unique),sizeof(char *)*((*uniqueNum)+1));
      }
      // Add this entry to unique list of programs
      (*unique)[(*uniqueNum)] = (char *)malloc(sizeof(char)*MAXLINE);
      strcpy((*unique)[(*uniqueNum)], e);
      (*uniqueNum) += 1;
   }

   return(isUnique);
}

// Check "missed" program against list of already "missed" programs
// Also make sure it does not match any scheduled recording.
// Include => return 1, otherwise return 0
int ensureUniqueMissed(
   char ***unique, int *uniqueNum, struct Channel *match,
   struct Channel **shows, int numShows )
{
char e[MAXLINE], p[MAXLINE];
int  isUnique=1;
int  i;

   // Make sure it's not a duplicate
   sprintf(e, "%d %s %s", match->channelNum, match->title, match->start);
   for( i=0; i<(*uniqueNum); ++i ) {
      if( !strcmp((*unique)[i],e) ) {
         // Identical program => not unique
         isUnique = 0;
      }
   }

   // Make sure it doesn't match a scheduled recording
   sprintf(e, "%d %s %s", match->channelNum, match->title, match->start);
   for( i=0; i<numShows; ++i ) {
      sprintf(p, "%d %s %s", shows[i]->channelNum, shows[i]->title, shows[i]->start);
      if( !strcmp(e,p) ) {
         isUnique = 0;
      }
   }

   if( isUnique ) {
      // Allocate more memory to store this new program in unique list
      if( *uniqueNum == 0 ) {
         (*unique) = (char **)malloc(sizeof(char *));
      } else {
         (*unique) = (char **)realloc((*unique),sizeof(char *)*((*uniqueNum)+1));
      }
      // Add this entry to unique list of programs
      (*unique)[(*uniqueNum)] = (char *)malloc(sizeof(char)*MAXLINE);
      strcpy((*unique)[(*uniqueNum)], e);
      (*uniqueNum) += 1;
   }

   return(isUnique);
}

// Given a program struct list (extracted from xmltv file) see if
// this program matches any of the extracted Replay channels.
// If so, return a channel struct with all relevant information.
// Note that progList contains up to 3 entries:
//    progList[0] == previous program
//    progList[1] == current program
//    progList[2] == next program
struct Channel *compareProgramToChannels(
   struct prog *progList,
   struct Channel **channels,
   int    numChan
)
{
int    i, j, match, aroundCandidates=0;
struct prog prev, program, next;
struct Channel *matchProg=NULL;
struct Channel *thisProg;

struct Around {
   struct prog program;
   struct prog prev;
   struct prog next;
   int    channelIndex;
} *around=NULL;

   prev = progList[0];
   program = progList[1];
   next = progList[2];

   for( i=0; i<numChan; ++i ) {
      match = 0;

      // Only channels with Days field set will look for recordings
      if ( strlen(channels[i]->Days) > 0 ) {

         if ( !strcmp(channels[i]->Type,"Single") ) {
            match = compareSingle(channels[i], &program);
         }
         else if ( !strcmp(channels[i]->Type,"Recurring") ) {
            // Rigorous test
            match = compareRecurring(channels[i], &program);

            if (match > 1) {
               // Add as "at or around" test candidate
               if( aroundCandidates == 0 ) {
                  around = (struct Around *)malloc(sizeof(struct Around));
               } else {
                  around = (struct Around *)realloc(
                     around,sizeof(struct Around)*(aroundCandidates+1)
                  );
               }
               around[aroundCandidates].program = program;
               if((match&2)==2) {
                  around[aroundCandidates].prev = prev;
               } else {
                  memset(&around[aroundCandidates].prev, 0, sizeof(prev));
               }
               if((match&4)==4) {
                  around[aroundCandidates].next = next;
               } else {
                  memset(&around[aroundCandidates].next, 0, sizeof(next));
               }
               around[aroundCandidates].channelIndex = i;
               aroundCandidates++;
            }

            // No match => see if we need to flag as missed recording
            if(match == 0) flagMissingHandler(channels[i], &program);
         }

         else if ( !strcmp(channels[i]->Type,"Theme") ) {
            match = compareTheme(channels[i], &program);
         }
         // Zone => Not enough info to process
         else if ( !strcmp(channels[i]->Type,"Zone") ) {
            match = 0;
         }
      } // if

      if ( match == 1 ) {
         // Perfect match, no need to continue
         thisProg = copyMatchedProgram(&program, channels[i]);
         if(matchProg == NULL) {
            matchProg = thisProg;
         } else {
            if(matchProg->Priority < thisProg->Priority) {
                // Higher priority than previous match - discard previous match
                free(matchProg);
                matchProg = thisProg;
            } else {
                // Lower priority than previous match - discard this match
                free(thisProg);
            }
         }

         // Set a flag indicating at least one match for this channel
         // so we can detect missing Single recordings
         channels[i]->matchcount++;

         //Keep looking for more matches, because the first isn't necessarily the best 
         //if(around) free(around);
         //return(matchProg);
      }

   } // for

   if(matchProg) {
	if(around) free(around);
	return matchProg;
   }

   // No perfect match found, so try "at or around" tests for candidates
   if( aroundCandidates > 0 ) {
      for( j=0; j<aroundCandidates; ++j ) {
         match = 0;
         program = around[j].program;
         prev = around[j].prev;
         next = around[j].next;
         i = around[j].channelIndex;
         // Check channel num and title of previous program
         if (strlen(prev.title) > 0) {
            if ( !strcmp(channels[i]->ChannelName, prev.channel)
		|| ((prev.tmsid != 0) && (channels[i]->tmsid == prev.tmsid)) ) {
               if ( strEqualStripped(channels[i]->Title,prev.title) || strEqualStripped(channels[i]->Title,prev.subtitle) ) {
                  match = -100;
                  // 5.0 software check for firstRun channel
                  if ( channels[i]->firstRun && prev.repeat ) {
                     match = 0;
                  }
               }
            }
         }

         // Check channel num and title of next program
         if ( (match != -100) && (strlen(next.title) > 0) ) {
            if ( !strcmp(channels[i]->ChannelName, next.channel)
		|| ((prev.tmsid != 0) && (channels[i]->tmsid == prev.tmsid)) ) {
               if ( strEqualStripped(channels[i]->Title,next.title) || strEqualStripped(channels[i]->Title,next.subtitle) ) {
                  match = 100;
                  // 5.0 software check for firstRun channel
                  if ( channels[i]->firstRun && next.repeat ) {
                     match = 0;
                  }
               }
            }
         }
         if( match != 0 ) {
            if ( match == -100 ) {
               matchProg = copyMatchedProgram(&prev, channels[i]);
               matchProg->AtOrAround = 1;
               channels[i]->matchcount++;
            }
            if ( match == 100 ) {
               matchProg = copyMatchedProgram(&next, channels[i]);
               matchProg->AtOrAround = 1;
               channels[i]->matchcount++;
            }
            if(around) free(around);
            return(matchProg);
         } else {
            // No match, so see if we need to flag as missed recording
            flagMissingHandler(channels[i], &program);
         }
      } // for
   }

   if(around) free(around);
   return(matchProg);
}

// Make a copy of the give matched program and add extra channel-related
// information to it and also compute and store it's priority.
// ReplayTV 4.x conflict resolution in order of priority:
// ------------------------------------------------------
// 1. Guaranteed single-record or show-based recurring channels
// 2. For single/single single/recurring recurring/recurring conflicts,
//    channel created first wins.
// 3. Guaranteed theme-based or zone-based channels
// 4. For theme/theme theme/zone zone/zone conflicts,
//    the show starting first wins or, if they start
//    at same time then the show airing on the lowest
//    channel number wins.
//   ---- Same pattern repeats for non-guaranteed channels ----
// Other reasons for a show NOT to be recorded:
//    - Channel is full (i.e. total time alloted is exceeded)
//       o Recurring/Theme/Zone based channels recycle a show only after 24
//         hours.
// ALGORITHM:
//   MainPriority =   0 if non-guaranteed
//   MainPriority = 100 if guaranteed
//   MainPriority += 11 if single
//   MainPriority += 10 if recurring
//   MainPriority +=  6 if theme
//   MainPriority +=  5 if zone
//
//   SubPriority = 1.0e8/(channel ustart) if single or recurring
//   SubPriority = 1/(program channel #) if theme or zone
//   Total Priority = MainPriority + SubPriority
//
//   if single or recurring
//      Highest Total Priority wins
//   if theme or zone
//      Show starting first wins OR if both start at same time:
//      Highest Total Priority wins
//
// ReplayTV 5.0 software conflict resolution in order of priority:
// ---------------------------------------------------------------
// 1. Single-record or show-based recurring channels take priority
//    over Theme & Zone channels.
// 2. Show with "Delete oldest: Only for new episode" has priority
//    over show with "Delete oldest: if space needed"
// 3. Channel created most recently has priority over older channel.
// ALGORITHM:
//   MainPriority = 0
//   MainPriority += 100 if single or recurring
//   MainPriority +=  10 if theme or zone
//   MainPriority +=   1 if (guaranteed) "Delete oldest: Only for new episode"
//
//   SubPriority = (channel ustart)/1.0e10 => Later has bigger priority
//   NOTE: SubPriority always < 1
//   Total Priority = MainPriority + SubPriority
//
//   Highest Total Priority wins
struct Channel *copyMatchedProgram(struct prog *program, struct Channel *channel) {
 struct Channel *matchProg=NULL;
 double  MainPriority, SubPriority;
   matchProg = (struct Channel *)malloc(sizeof(struct Channel));
   resetChannelStruct(matchProg);
   copyProgToChan(matchProg, program);
   // Add extra info to matchProg hash
   strcpy(matchProg->Type, channel->Type);
   strcpy(matchProg->Name, channel->Name);
   strcpy(matchProg->ChannelDate, channel->ChannelDate);
   strcpy(matchProg->Title, channel->Title);
   strcpy(matchProg->Days, channel->Days);
   strcpy(matchProg->Keywords, channel->Keywords);
   strcpy(matchProg->SearchKeys, channel->SearchKeys);
   matchProg->Minutes = channel->Minutes;
   strcpy(matchProg->ShowFlags,channel->ShowFlags);
   matchProg->PaddingBefore = channel->PaddingBefore;
   matchProg->PaddingAfter = channel->PaddingAfter;
   strcpy(matchProg->ChannelName, channel->ChannelName);
   matchProg->tmsid = channel->tmsid;
   strcpy(matchProg->ip, channel->ip);
   strcpy(matchProg->Time, channel->Time);
   matchProg->Guaranteed = channel->Guaranteed;
   matchProg->PriorityBoost = channel->PriorityBoost;
   matchProg->firstRun = channel->firstRun;
   matchProg->channelID = channel->channelID;
   strcpy(matchProg->CategoryName, channel->CategoryName);
   if( ! matchProg->ustop ) {
      matchProg->ustop = matchProg->ustart + channel->Minutes*60;
   }
   // Manual record stop time controlled by channel Minutes
   if( !strcmp(channel->ShowFlags, "Manual record") ) {
      matchProg->ustop = guideTimeToUTC(channel->Time) + channel->Minutes*60;
   }

   // Deal with time padding (given in minutes)
   matchProg->ustart -= 60*channel->PaddingBefore;
   matchProg->ustop += 60*channel->PaddingAfter;

   // Compute priority number for this program
   MainPriority = 0;
   if( RTV_OS == 5 ) {
      // The PriorityBoost flag increases a shows priority
      // and overrides the ChannelDate value
      // Normally this value is 0, but it gets incremented
      // every time you force a channel to increase scheduling priority
      // Things are scaled so we can get to 999 before overflow
      // I'm guessing we'll never get up to 999 on this counter
      SubPriority = 1.0 + matchProg->PriorityBoost;
      if( !strcmp(matchProg->Type,"Single") ) MainPriority += 1e4*SubPriority;
      if( !strcmp(matchProg->Type,"Recurring") ) MainPriority += 1e4*SubPriority;
      if( !strcmp(matchProg->Type,"Theme") ) MainPriority += 10*SubPriority;
      if( !strcmp(matchProg->Type,"Zone") ) MainPriority += 10*SubPriority;
      if( matchProg->Guaranteed != 0 ) MainPriority += 1;
      SubPriority = guideTimeToUTC(matchProg->ChannelDate)/1.0e10f;
   } else {
      if( matchProg->Guaranteed != 0 ) MainPriority += 100;
      if( !strcmp(matchProg->Type,"Single") ) MainPriority += 11;
      if( !strcmp(matchProg->Type,"Recurring") ) MainPriority += 10;
      if( !strcmp(matchProg->Type,"Theme") ) MainPriority += 6;
      if( !strcmp(matchProg->Type,"Zone") ) MainPriority += 5;

      SubPriority = 1.0e8f/guideTimeToUTC(matchProg->ChannelDate);
      if( !strcmp(matchProg->Type,"Theme") || !strcmp(matchProg->Type,"Zone")) {
         SubPriority = 1.0f/matchProg->channelNum;
      }
   }

   matchProg->Priority = MainPriority + SubPriority;
   matchProg->PriorityLoser = 0; // just to initialize

   return(matchProg);
}

// Check to see if we need to flag this program as not matching
// a Recurring Channel with same title and channel name. This is
// to catch programs you may want to watch but are not currently
// scheduled to record.
int flagMissingHandler(struct Channel *chan, struct prog *program)
{
   if( flagMissing ) {
      if( strEqualStripped(chan->Title,program->title) || strEqualStripped(chan->Title,program->subtitle) ) {
         if( (!strcmp(chan->ChannelName,program->channel)
              || ((chan->tmsid != 0) && (chan->tmsid == program->tmsid))) &&
             flagThisChannel(chan->ChannelName) && ! ignoreThisMissedShow(program->title,program->repeat) ) {
            if( NUM_MISSING == 0 ) {
               MISSING = (struct Channel **)malloc(sizeof(struct Channel *));
            } else {
               MISSING = (struct Channel **)realloc(
                  MISSING, sizeof(struct Channel *)*(NUM_MISSING+1)
               );
            }
            MISSING[NUM_MISSING] = (struct Channel *)malloc(
               sizeof(struct Channel)
            );
            resetChannelStruct(MISSING[NUM_MISSING]);
            copyProgToChan(MISSING[NUM_MISSING], program);
            NUM_MISSING++;
         }
      }
   }
   return(0);
}

// Return 1 if looking to flag missing programs for this channel,
// return 0 otherwise.
int flagThisChannel(char name[]) {
 char prefixed[64], suffixed[64];

   // Flag all but specified channels
   if( CONFIG.htmlMissingChannels[0] == '!' ) {
      // Single channel to be not flagged
      sprintf(prefixed, "!%s", name);
      if( !strcmp(CONFIG.htmlMissingChannels,prefixed) ) {
         return(0);
      }

      // Look for specific channel name in comma separated list
      sprintf(prefixed, ",%s", name);
      sprintf(suffixed, "%s,", name);
      if( strstr(CONFIG.htmlMissingChannels,prefixed) ||
          strstr(CONFIG.htmlMissingChannels,suffixed) ) {
         return(0);
      }
      return(1);
   }

   // Flag specified channels only
   else {
      // If flagging all channels then always true
      if( !strcmp(CONFIG.htmlMissingChannels,"ALL") ) {
         return(1);
      }

      // Single channel to be flagged matches the one passed
      if( !strcmp(CONFIG.htmlMissingChannels,name) ) {
         return(1);
      }

      // Look for specific channel name in comma separated list
      sprintf(prefixed, ",%s", name);
      sprintf(suffixed, "%s,", name);
      if( strstr(CONFIG.htmlMissingChannels,prefixed) ||
          strstr(CONFIG.htmlMissingChannels,suffixed) ) {
         return(1);
      }
      return(0);
   }
}

// Return 1 if this show name should be ignored when
// building list of potentially missed shows
int ignoreThisMissedShow(char name[], int repeat) {
 char prefixed[256], suffixed[256];

   // Single entry match
   if( !strcmp(CONFIG.htmlMissingShowIgnore,name) ) {
      return(1);
   }

   // Look for specific channel name in comma separated list
   sprintf(prefixed, ",%s", name);
   sprintf(suffixed, "%s,", name);
   if( strstr(CONFIG.htmlMissingShowIgnore,prefixed) ||
       strstr(CONFIG.htmlMissingShowIgnore,suffixed) ) {
      return(1);
   }

   // Ignore if a repeat and option to ignore repeats is enabled
   if( CONFIG.htmlDontFlagMissingRepeats && repeat ) {
      return(1);
   }

   return(0);
}

// Single => Compare:
//                channel name
//                starting year
//                starting month
//                starting day
//                starting hour
//                starting minute
int compareSingle(struct Channel *chan, struct prog *program) {
   int match = 0;
   struct Time progTime, chanTime;

   if ( !strcmp(chan->ChannelName, program->channel) 
        || ((chan->tmsid != 0) && (chan->tmsid == program->tmsid)) ) {
      progTime = guideTimeToLocalTime(program->start);
      chanTime = guideTimeToLocalTime(chan->Time);
      if (
         progTime.year  == chanTime.year &&
         !strcmp(progTime.month, chanTime.month) &&
         progTime.mday  == chanTime.mday &&
         progTime.hour  == chanTime.hour &&
         progTime.min   == chanTime.min
      ) {
         match = 1;
         /* Used for debugging matching for a specific show
         if(DEBUG
	    &&  (    (strcmp(program->title,"Keeping Up Appearances")==0)
	          || (strcmp(chan->Title,   "Keeping Up Appearances")==0) ) ) {
	        fprintf(stderr, "%s %d '%s', %s %d '%s' p=%d:%02d, c=%d:%02d+%d\n",
	        program->channel, program->tmsid, program->title,
		chan->channel, chan->tmsid, chan->Title,
	        progTime.hour, progTime.min,
	        chanTime.hour, chanTime.min, chan->Minutes);
         }*/
      }
   }
   return(match);
}

// Recurring => Compare:
//                channel name
//                program title and/or sub-title
//                starting hour
//                starting minute
//                day of week (Sun-Sat)
//                firstRun channel && repeat program => no match (5.0 software)
//                "at or around" (+/- 1 time slot) concept applies
//
// chan = data from the Replay Guide
// program = data from the channel guide

int compareRecurring(struct Channel *chan, struct prog *program) {
   int match = 0;
   int channelMatch=0, wdayMatch=0, titleMatch=0, timeOverlap=0, dayOffset=0;
   struct Time progTimeStart, progTimeStop, chanTime;
   int pstart, pstop, cstart, cstop;
   int wday;

   if ( !strcmp(chan->ChannelName, program->channel) 
        || ((chan->tmsid != 0) && (chan->tmsid == program->tmsid)) ) {
      channelMatch = 1;
   }

   // Don't bother proceeding if channel names don't match
   if ( ! channelMatch ) {
      return(0);
   }

   // Calculate program start & stop in seconds from midnight
   progTimeStart = guideTimeToLocalTime(program->start);
   pstart = progTimeStart.hour*60*60 + progTimeStart.min*60;
   if( program->ustop ) {
      progTimeStop  = utcToLocalTime(program->ustop); //guideTimeToLocalTime(program->stop);
      pstop = progTimeStop.hour*60*60 + progTimeStop.min*60;
   } else {
      // Fabricate a stop time since XMLTV didn't give us one
      pstop = pstart + chan->Minutes*60;
   }
   // Catch the 24-hour wrapping
   if( pstop < pstart  ) {
       pstop = pstop + 24*60*60;
   }

   // Calculate channel start & stop in seconds from midnight
   chanTime = guideTimeToLocalTime(chan->Time);
   cstart = chanTime.hour*60*60 + chanTime.min*60;
   cstop = cstart + chan->Minutes*60;

   // Check if channel and program times overlap
   // Program is a show from the tv listings
   // Channel is the replaytv's selection we are trying to match
   if ( pstart < cstop && pstop > cstart ) {
      timeOverlap = 1;
      dayOffset = 0;
   }
   // Check for a time match offset one day earlier
   else if( pstart < (cstop - 24*60*60) && pstop > (cstart - 24*60*60) ) {
      timeOverlap = 1;
      // Must decrease program day-of-week by one for compare with channel days
      dayOffset = -1;
   }
   // Check for a time match offset one day later
   else if( pstart < (cstop + 24*60*60) && pstop > (cstart + 24*60*60) ) {
      timeOverlap = 1;
      // Must increase program day-of-week by one for compare with channel days
      dayOffset = +1;
   }
   // Get the name of the day, including any offset
   wday = progTimeStart.wdayindex + dayOffset;
   if(wday < 0) wday += 7;
   if(wday >= 7) wday -= 7;
   if((wday < 0) || (wday >= 7)) wday = 0;

   // Check if channel weekday matches program weekday
   if( hasStr(chan->Days, DayName[wday], 1) ) {
      wdayMatch = 1;
   }

   // Check for recurring Manual Record
   if ( wdayMatch && hasStr(chan->ShowFlags, "Manual record", 0) ) {
      // If manual recording starts within this program
      // then fake things out so this program matches with the correct length
      // so that the conflict detector can handle it properly
      if ( cstart >= pstart && cstart < pstop ) {
         match = 1;
         // Fake out the start/stop mismatch with (possibly negative) padding
         chan->PaddingBefore = (pstart-cstart)/60;
         if(pstop >= cstop) chan->PaddingAfter = (cstop-pstop)/60;
         if(DEBUG) {
            printf("*** Faking out manual record padding:\n");
            printf("*** pstart = %d, pstop = %d\n", pstart/3600, pstop/3600);
            printf("*** cstart = %d, cstop = %d\n", cstart/3600, cstop/3600);
            printProgramStruct(program);
            printf("\n");
            printChannelInfo(chan);
         }
      }
   }

   // Check if channel and program titles match
   if ( strEqualStripped(chan->Title,program->title) || strEqualStripped(chan->Title,program->subtitle) ) {
      titleMatch = 1;
   }

   if(DEBUG && wdayMatch && (match || titleMatch)) {
      if(dayOffset < 0) {
         printf("*** Time overlap one day earlier\n");
         printProgramStruct(program);
         printf("\n");
         printChannelInfo(chan);
      } else if(dayOffset > 0) {
         printf("*** Time overlap one day later\n");
         printProgramStruct(program);
         printf("\n");
         printChannelInfo(chan);
      }
   }

   /* Used for debugging matching for a specific show
   if(DEBUG && (titleMatch||timeOverlap) &&
	(    (strcmp(program->title,"Mulheres Apaixonadas")==0)
	  || (strcmp(chan->Title,   "Mulheres Apaixonadas")==0) ) ) {
	fprintf(stderr, "'%s','%s' p=%s %d %d:%02d-%d:%02d (%d-%d), c=%d:%02d+%d (%d-%d), title=%d, wday=%d, overlap=%d, offset=%d\n",
	program->title, chan->Title,
	progTimeStart.month, progTimeStart.mday,
	progTimeStart.hour, progTimeStart.min, progTimeStop.hour, progTimeStop.min, pstart, pstop,
	chanTime.hour, chanTime.min, chan->Minutes, cstart, cstop,
	titleMatch, wdayMatch, timeOverlap, dayOffset);
   }*/

   if( ! match ) {
      // Still no match so check for channel/program time overlap
      if( wdayMatch ) {

         if( timeOverlap ) {
            // Current program overlaps channel time slot
            if ( titleMatch ) {
               // Program overlaps channel time slot & titles match
               match = 1;
            } else {
               // Program overlaps channel time slot but no title match
               // so try "at or around test"
               // Set bit 1 (value 2) if we want to check prev show for at-or-around
               // Set bit 2 (value 4) if we want to check next show for at-or-around
               // This is used so that we don't bother with at-or-around checks when
               // a time overlap match will catch the show
               // P is a show from the tv listings
               // C is the replaytv's selection we are trying to match
               if ( pstart <= (cstart + 24*60*60*dayOffset) )
                  match |= 2;  // Need to check previous time slot for at-or-around
               if ( pstop >= (cstop + 24*60*60*dayOffset) )
                  match |= 4;  // Need to check next time slot for at-or-around
            }
         }
      }
   }

   if( match == 1 ) {
      // firstRun channel && repeat program => no match (5.0 software)
      if( chan->firstRun && program->repeat ) {
         match = 0;
      }
   }

   return(match);
}

// Theme => Look for Channel keywords in certain program fields
//          according to chan->SearchKeys
int compareTheme(struct Channel *chan, struct prog *program) {

   int match = 0;

   if ( hasStr(chan->SearchKeys, "PartialTitle", 1) ) {
      if (
         hasStr(stripNonAlpha(program->title), chan->Keywords,2) ||
         hasStr(stripNonAlpha(program->subtitle), chan->Keywords,2)
      ) {
         match = 1;
      }
   }

   if ( hasStr(chan->SearchKeys, "ExactTitle", 1) ) {
      if (
         !strcmp(program->title, chan->Keywords)
      ) {
         match = 1;
      }
   }

   if ( hasStr(chan->SearchKeys, "Description", 1) ) {
      if (
         hasStr(stripNonAlpha(program->desc), chan->Keywords,2) ||
         hasStr(stripNonAlpha(program->subtitle), chan->Keywords,2)
      ) {
         match = 1;
      }
   }

   if ( hasStr(chan->SearchKeys, "Actor", 1) ) {
      if ( strlen(program->actors) > 0 ) {
         if ( hasStr(stripNonAlpha(program->actors), chan->Keywords,2) ) {
            match = 1;
         }
      } else {
         if (
            hasStr(stripNonAlpha(program->desc), chan->Keywords,2) ||
            hasStr(stripNonAlpha(program->subtitle), chan->Keywords,2)
         ) {
            match = 1;
         }
      }
   }

   if ( hasStr(chan->SearchKeys, "Director", 1) ) {
      if ( strlen(program->directors) > 0 ) {
         if ( hasStr(stripNonAlpha(program->directors), chan->Keywords,2) ) {
            match = 1;
         }
      } else {
         if (
            hasStr(stripNonAlpha(program->desc), chan->Keywords,2) ||
            hasStr(stripNonAlpha(program->subtitle), chan->Keywords,2)
         ) {
            match = 1;
         }
      }
   }

   return(match);
}

// Strip off non-alpha characters (except spaces) and lowercase
char *stripNonAlpha(char *string) {
int i, j, add;
char c;
	j=0;
	for( i=0; i<strlen(string); ++i ) {
	   add = 1;
	   c = string[i];
	   // Non-alphanumeric and non-space character are skipped
       if( (c != ' ') && ! isalnum(c) ) {
	      switch( c ) {
	       case '-':
	       case '/':
	         c = ' '; break;
               default:
	         add = 0; break;
	      }
	   }

           // Avoid 2 spaces in a row
           if( j > 0 && (c == ' ') && (_string_[j-1]==' ') ) {
	      add = 0;
	   }

	   if( add ) {
	       _string_[j++] = tolower(c);
	   }
	}
	_string_[j] = '\0';
	return(_string_);
}

// Return 1 if two non-alpha stripped strings are identical
int strEqualStripped(char *s1, char *s2) {
char str1[MAXLINE], str2[MAXLINE];
   strcpy(str1,stripNonAlpha(s1));
   strcpy(str2,stripNonAlpha(s2));
   if( !strcmp(str1,str2) )
      return 1;
   else
      return 0;
}

// Return 1 if two strings are identical minus any non-alpha characters
// and minus any of the words "a" "an" "the" or "and"
// This version commented out because it proved not to work:
// For example: "The Next Action Star" vs. "Next Action Star" the RTV
// recurring channel did NOT record because of this title mismatch, so
// stripping out words is not the right algorithm.
/*
int strEqualStripped(char *s1, char *s2) {
    int idx1,idx2,base1,base2,ch1,ch2,skip;
    char str1[MAXLINE];
    char str2[MAXLINE];
    strcpy(str1,stripNonAlpha(s1));
    strcpy(str2,stripNonAlpha(s2));
    
    idx1=0;
    idx2=0;

//fprintf(stderr,"Comparing '%s' and '%s'\n   ",s1,s2);

    while(str1[idx1] && str2[idx2]) {
        // Parse one word at a time
        do {
            for(base1=idx1; str1[idx1] && !isspace(str1[idx1]); idx1++);
            // Skip over any words in the ignore list
            ch1=str1[idx1];
            str1[idx1]=0;
            skip=0;
            if(!strcmp(str1+base1,"a")) skip=1;
            else if(!strcmp(str1+base1,"an")) skip=1;
            else if(!strcmp(str1+base1,"and")) skip=1;
            else if(!strcmp(str1+base1,"the")) skip=1;
            if(skip && ch1) {
                //fprintf(stderr,"(%s) ",str1+base1);
                idx1++;
            }
        } while(skip && ch1);

        do {
            for(base2=idx2; str2[idx2] && !isspace(str2[idx2]); idx2++);
            // Skip over any words in the ignore list
            ch2=str2[idx2];
            str2[idx2]=0;
            skip=0;
            if(!strcmp(str2+base2,"a")) skip=1;
            else if(!strcmp(str2+base2,"an")) skip=1;
            else if(!strcmp(str2+base2,"and")) skip=1;
            else if(!strcmp(str2+base2,"the")) skip=1;
            if(skip && ch2) {
                //fprintf(stderr,"(%s)",str2+base2);
                idx2++;
            }
        } while(skip && ch2);

	if(strcmp(str1+base1,str2+base2)) {
            //fprintf(stderr, "%s != %s  NOT EQUAL\n",str1+base1,str2+base2);
            return 0;
        }

        //fprintf(stderr,"%s==%s  ",str1+base1,str2+base2);
        if(ch1) idx1++;
        if(ch2) idx2++;
    }
    // Return true if both titles are at the end of the string
    // One of these must be zero
    // The strings are not equal if one of these values is non-zero
    // Note we never strip words at the end of the string
    if(str1[idx1] == str2[idx2]) {
        //fprintf(stderr, "EQUAL\n");
        return 1;
    } else {
        //fprintf(stderr, "NOT EQUAL\n");
        return 0;
    }
}
*/

// Check for time overlaps of any recordings. If conflicts
// found mark the appropriate recordings as conflicting, and
// store information about which shows it conflicts with.
void checkForConflicts( struct Channel ***recordings, int numProg ) {
int i, j, conflict;
struct Channel *a, *b;

   if(!quiet) fprintf(stderr,">> Checking for schedule conflicts ...\n");

   // Check UTC numbers for any overlap
   for ( i=0; i<numProg; i++ ) {
      a = (*recordings)[i];
      for ( j=0; j<numProg; j++ ) {
         if( i != j ) {
            b = (*recordings)[j];
            conflict = 0;

            // If both start at same time, that's a conflict
            if ( a->ustart == b->ustart ) {
               if ( !strcmp(a->ip, b->ip) )
                  conflict = 1;
            }

            // If time ranges intersect, that's a conflict
            if ( a->ustart < b->ustart ) {
               if ( a->ustop > b->ustart ) {
                  if ( !strcmp(a->ip, b->ip) )
                     conflict = 1;
               }
            }
            if ( b->ustart < a->ustart ) {
               if ( b->ustop > a->ustart ) {
                  if ( !strcmp(a->ip, b->ip) )
                     conflict = 1;
               }
            }

            // Oh oh, conflict found
            if ( conflict ) {
              if(!quiet) {
                 printf("\n** Conflict found **\n");
                 printProgramInfo(a);
                 printf("\n");
                 printProgramInfo(b);
              }

              a->conflict = 1;
              b->conflict = 1;

              // Determine the priority loser between these 2

              // At or around match gets less priority
              if( a->AtOrAround && !b->AtOrAround ) {
                 // a is loser
                 a->PriorityLoser = 1;
              }
              if( !a->AtOrAround && b->AtOrAround ) {
                 // b is loser
                 b->PriorityLoser = 1;
              }

              if( !a->PriorityLoser && !b->PriorityLoser ) {
               if ( RTV_OS == 4 && (!strcmp(a->Type,"Theme") || !strcmp(a->Type,"Zone")) && (!strcmp(b->Type,"Theme") || !strcmp(b->Type,"Zone")) ) {
                  // Theme/Zone Theme/Theme Zone/Zone conflict resolution
                  if ( a->ustart < b->ustart ) {
                     // b is loser
                     b->PriorityLoser = 1;
                  }
                  if ( b->ustart < a->ustart ) {
                     // a is loser
                     a->PriorityLoser = 1;
                  }
                  if ( a->ustart == b->ustart ) {
                     if ( a->Priority > b->Priority ) {
                        // b is loser
                        b->PriorityLoser = 1;
                     } else {
                        // a is loser
                        a->PriorityLoser = 1;
                     }
                  }
               } else {
                  // All other comparisons go strictly by priority
                  if ( a->Priority > b->Priority ) {
                     // b is loser
                     b->PriorityLoser = 1;
                  } else {
                     // a is loser
                     a->PriorityLoser = 1;
                  }
               }
              }

               // Save modified/added info back to recordings array
               (*recordings)[i] = a;
               (*recordings)[j] = b;
            }
         }
      }
   }

   // 1 more pass
   // Check each priority loser against all priority winners and see
   // if it still conflicts with the remaining shows.
   for ( i=0; i<numProg; i++ ) {
      a = (*recordings)[i];
      // If a is conflicting and a priority loser check against winners
      // If no conflict with winners then mark as priority winner
      if ( a->conflict && a->PriorityLoser ) {
         conflict = 0;
         for ( j=0; j<numProg; j++ ) {
            if( i != j ) {
               b = (*recordings)[j];
               if( ! b->PriorityLoser ) {

                  // If both start at same time, that's a conflict
                  if ( a->ustart == b->ustart ) {
                     if ( !strcmp(a->ip, b->ip) )
                        conflict = 1;
                  }

                  // If time ranges intersect, that's a conflict
                  if ( a->ustart < b->ustart ) {
                     if ( a->ustop > b->ustart ) {
                        if ( !strcmp(a->ip, b->ip) )
                           conflict = 1;
                     }
                  }
                  if ( b->ustart < a->ustart ) {
                     if ( b->ustop > a->ustart ) {
                        if ( !strcmp(a->ip, b->ip) )
                           conflict = 1;
                     }
                  }
               }
            }
         }
         if( ! conflict ) {
            (*recordings)[i]->PriorityLoser = 0;
         }
      }
   }
}

// Sort programs in chronological order by start time
void sortProgs( struct Channel ***sorted, struct Channel ***progs, int numProg ) {
int i, j, *order, ignore, index, num=0;
time_t low;

   order = (int *)malloc(sizeof(int)*numProg);
   (*sorted) = (struct Channel **)malloc(sizeof(struct Channel *)*numProg);

   // Initialize order list
   for( i=0; i<numProg; ++i ) {
      order[i] = -1;
   }

   // Build order list
   while( num < numProg ) {
      index = -1;
      low = -1;
      for( i=0; i<numProg; ++i ) {
         ignore = 0;
         // Don't consider this entry if already handled
         for( j=0; j<num; ++j ) {
            if( order[j] == i ) {
               ignore = 1;
            }
         }
         if( !ignore ) {
            if( low == -1 ) {
               low = (*progs)[i]->ustart;
               index = i;
            }
            if( (*progs)[i]->ustart < low ) {
               low = (*progs)[i]->ustart;
               index = i;
            }
         }
      }
      if( index != -1 ) {
         order[num++] = index;
      }
   }

   // Create new sorted order of programs
   for( i=0; i<num; ++i ) {
      (*sorted)[i] = (*progs)[order[i]];
   }

   free(order);
}

int createXmlSchedule(char *xmlFile, char *replayIP,
  struct Channel **shows, int numShows,
  int *CategoryIndex, char **CategoryName, int numCategory, int noTheme) {
  FILE *XML;
  int i, skip;
  time_t now=time(NULL);
  struct tm * t=gmtime(&now);

  if(!quiet) fprintf(stderr,">> Creating xml file: '%s' ...\n", xmlFile);

  // Map "-" to stdout
  if((xmlFile[0] == '-') && (xmlFile[1] == 0)) {
    XML = stdout;
  } else {
    XML = fopen(xmlFile, "w");
    if(XML == NULL) {
       fprintf(stderr, "*ERROR* Failed to write to file: '%s'\n", xmlFile);
       return 0;
    }
  }
  // Start out with a standard header
  fprintf(XML, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
  fprintf(XML, "<!DOCTYPE schedule SYSTEM \"replayschedule.dtd\">\n");
  fprintf(XML, "\n<schedule date=\"%04d%02d%02d%02d%02d%02d %+05d\" ",
	t->tm_year+1900, t->tm_mon+1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec, 0);
  fprintf(XML, "source-info-url=\"http://%s/http_replay_guide-get_snapshot\" ",replayIP);
  fprintf(XML, "source-info-name=\"ReplayTV\" ");
  fprintf(XML, "generator-info-name=\"replaySchedule\" ");
  //fprintf(XML, "generator-info-url=\"http:dont.have.any.yet\" ");
  fprintf(XML, ">\n");

  // Write specific unit version information
  fprintf(XML, " <replayosversion>%d.%d</replayosversion>\n", 3+guideversion, replayversion);
  fprintf(XML, " <replayversion>%d</replayversion>\n", replayversion);
  fprintf(XML, " <guideversion>%d</guideversion>\n", guideversion);
  // Write the list of categories
  for( i=0; i<numCategory; i++) {
     fprintf(XML, "  <category id=\"%d\">%s</category>\n",
        CategoryIndex[i], CategoryName[i]);
  }
  for( i=0; i<numShows; ++i ) {
    skip = 0;
    if( noTheme && !strcmp(shows[i]->Type,"Theme") ) {
       skip = 1;
    }
    if( ! skip ) {
      if(shows[i]->stop[0] == '\0') {
         if(shows[i]->tmsid != 0) {
            fprintf(XML, "  <programme start=\"%s\" tmsid=\"%u\" channel=\"%d %s\">\n",
               shows[i]->start, shows[i]->tmsid, shows[i]->channelNum, shows[i]->channel);
         } else {
            fprintf(XML, "  <programme start=\"%s\" channel=\"%d %s\">\n",
               shows[i]->start, shows[i]->channelNum, shows[i]->channel);
         }
      } else {
         if(shows[i]->tmsid != 0) {
            fprintf(XML, "  <programme start=\"%s\" stop=\"%s\" tmsid=\"%u\" channel=\"%d %s\">\n",
               shows[i]->start, shows[i]->stop, shows[i]->tmsid,
               shows[i]->channelNum, shows[i]->channel);
         } else {
            fprintf(XML, "  <programme start=\"%s\" stop=\"%s\" channel=\"%d %s\">\n",
               shows[i]->start, shows[i]->stop, shows[i]->channelNum, shows[i]->channel);
         }
      }
      if(shows[i]->title[0] != '\0')
         fprintf(XML, "    <title>%s</title>\n", shows[i]->title);
      if(shows[i]->subtitle[0] != '\0')
         fprintf(XML, "    <sub-title>%s</sub-title>\n", shows[i]->subtitle);
      fprintf(XML, "    <address>%s</address>\n", replayIP);
      if(shows[i]->PaddingBefore != 0)
         fprintf(XML, "    <pad-before>%d</pad-before>\n", shows[i]->PaddingBefore);
      if(shows[i]->PaddingAfter != 0)
         fprintf(XML, "    <pad-after>%d</pad-after>\n", shows[i]->PaddingAfter);
      fprintf(XML, "    <type>%s</type>\n",shows[i]->Type);
      fprintf(XML, "    <guaranteed>%d</guaranteed>\n", shows[i]->Guaranteed);
      fprintf(XML, "    <manual>%d</manual>\n",
         hasStr(shows[i]->ShowFlags, "Manual record", 0));
      fprintf(XML, "    <firstrun>%d</firstrun>\n", shows[i]->firstRun);
      if(shows[i]->Time[0])
         fprintf(XML, "    <created>%s</created>\n", shows[i]->Time);
      if(shows[i]->PriorityBoost)
         fprintf(XML, "    <boost>%d</boost>\n", shows[i]->PriorityBoost);
      if ( shows[i]->conflict ) {
         if ( shows[i]->PriorityLoser ) {
            fprintf(XML, "    <conflict>Loser</conflict>\n");
         } else {
            fprintf(XML, "    <conflict>Winner</conflict>\n");
         }
         //fprintf(XML, "    <priority>%g</priority>\n", shows[i]->Priority);
      }
      if(shows[i]->channelID != 0)
         fprintf(XML, "    <ReplayChannel>%d</ReplayChannel>\n", shows[i]->channelID);
      fprintf(XML, "  </programme>\n");
    }
  }
  // Add the definition termination
  fprintf(XML, "</schedule>\n");
  // If mapped to stdout, no need to close output file
  if((xmlFile[0] != '-') || (xmlFile[1] != 0))
     fclose(XML);
   return 1;
}

int createHtmlSchedule(
   char htmlFile[], char replayIP[MAX_IP_NUM][256], int ipCount,
   struct Channel **shows, int numShows,
   int noTheme,
   struct Channel **singleNoMatches, int numSingleNoMatch,
   struct Channel **emptyChannels, int numEmpty)
{
FILE *HTML;
struct Time today, start, stop;
struct Channel *s;
char string[MAXLINE], s1[50], s2[50], day[50], **days=NULL;
char repeatStr[20];
int  i, numDays, skip;
char **unique=NULL;
int  uniqueNum=0;
time_t utc;

   if(!quiet) fprintf(stderr,">> Creating html file: '%s' ...\n", htmlFile);

  // Map "-" to stdout
  if((htmlFile[0] == '-') && (htmlFile[1] == 0)) {
    HTML = stdout;
  } else {
   HTML = fopen(htmlFile, "w");
   if( ! HTML ) {
      fprintf(stderr, "*ERROR* Failed to write to file: '%s'\n", htmlFile);
      return(0);
   }
  }

  today = utcToLocalTime(time(NULL));
  sprintf(string, "Recording Schedule for Replay(s):");
  for(i=0; i<ipCount; ++i) {
     strcat(string, "&nbsp;\"");
     strcat(string, getReplayName(replayIP[i]));
     strcat(string, "\"");
     if( i != ipCount-1 )
        strcat(string, ",");
  }
  fprintf(HTML, "<!DOCTYPE HTML PUBLIC>\n");
  fprintf(HTML, "<html><body bgcolor=\"%s\">\n", CONFIG.htmlBackgroundColor);
  fprintf(HTML, "<title>%s</title>\n",string);
  fprintf(HTML, "<P align=\"left\">\n");
  fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
  fprintf(HTML, "<b>Today is: %s %s %d %d</b>\n", today.wday, today.month, today.mday, today.year);
  fprintf(HTML, "<br>\n");
  fprintf(HTML, "<b>%s</b></font>&nbsp;&nbsp;",string);
  fprintf(HTML, "<font color=\"%s\"><b>%s</b></font>",
     CONFIG.htmlConflictLoserColor,
     CONFIG.htmlConflictLoserTitle
  );
  fprintf(HTML, "<font color=\"%s\">&nbsp;<b>%s</b></font>",
     CONFIG.htmlConflictWinnerColor,
     CONFIG.htmlConflictWinnerTitle
  );
  if( numSingleNoMatch > 0 ) {
     fprintf(HTML, "<br><font color=\"%s\">NOTE:&nbsp;%s</font>\n",
        CONFIG.htmlSingleNoMatchColor, CONFIG.htmlSingleNoMatchTitle
     );
  }
  fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
  if( strlen(CONFIG.htmlSingleChannelTag) > 0 ) {
     fprintf(HTML, "<br>\n");
     fprintf(HTML, "%s&nbsp;=>&nbsp;Single Record Channel\n",
         CONFIG.htmlSingleChannelTag
     );
  }
  if( strlen(CONFIG.htmlRecurringChannelTag) > 0 ) {
     fprintf(HTML, "<br>\n");
     fprintf(HTML, "%s&nbsp;=>&nbsp;Recurring Record Channel\n",
         CONFIG.htmlRecurringChannelTag
     );
  }
  if( strlen(CONFIG.htmlRecurringChannelAtOrAroundTag) > 0 ) {
     fprintf(HTML, "<br>\n");
     fprintf(HTML, "%s&nbsp;=>&nbsp;Recurring Record Channel At or Around Match\n",
         CONFIG.htmlRecurringChannelAtOrAroundTag
     );
  }
  if( strlen(CONFIG.htmlThemeChannelTag) > 0 ) {
     fprintf(HTML, "<br>\n");
     fprintf(HTML, "%s&nbsp;=>&nbsp;Theme Channel\n",
         CONFIG.htmlThemeChannelTag
     );
  }
  numDays = getDaysInShows(&days, shows, numShows);
  strcpy(day,"");
  for( i=0; i<numShows; ++i ) {
     s = shows[i];
     skip = 0;
     if( noTheme && !strcmp(s->Type,"Theme") ) {
        skip = 1;
     }
     start = utcToLocalTime(s->ustart);
     stop = utcToLocalTime(s->ustop);

     if( ! skip ) {
      // Day Links block
      sprintf(s1, "%s %d (%s)", start.month, start.mday, start.wday);
      if ( strcmp(s1,day) ) {
         strcpy(day,s1);
         strcpy(string, buildDayLinks(day,days,numDays));
         fprintf(HTML, "<br><br><br><br>\n");
         fprintf(HTML, "%s\n",string);
         fprintf(HTML, "<br>\n");
      }

      // Show info block
      fprintf(HTML, "<hr width=\"100%%\">\n");
      if ( s->conflict ) {
         if ( s->PriorityLoser ) {
            fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlConflictLoserColor);
         } else {
            fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlConflictWinnerColor);
         }
      } else {
         fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlForegroundColor);
      }
      strcpy(repeatStr, "");
      if( s->repeat ) {
         strcpy(repeatStr, " (Repeat)");
      }
      strcpy(s1,"");
      if( !strcmp(s->Type,"Single") ) {
         strcpy(s1, CONFIG.htmlSingleChannelTag);
      }
      if( !strcmp(s->Type,"Recurring") ) {
         if( s->AtOrAround ) {
            strcpy(s1, CONFIG.htmlRecurringChannelAtOrAroundTag);
         } else {
            strcpy(s1, CONFIG.htmlRecurringChannelTag);
         }
      }
      if( !strcmp(s->Type,"Theme") ) {
         strcpy(s1, CONFIG.htmlThemeChannelTag);
      }
      if( ipCount > 1 ) fprintf(HTML, "%s\n<br>\n", getReplayName(s->ip));
      fprintf(HTML, "%s&nbsp;%d (%s)&nbsp;&nbsp;&nbsp;<b>%s%s</b>",
         s1, s->channelNum, s->channel, s->title, repeatStr
      );
      if( strlen(s->subtitle) > 0 ) {
         fprintf(HTML, " - %s", s->subtitle);
      }
      fprintf(HTML, "<br>\n");
      sprintf(s1, "%d:%d", start.hour, start.min);
      sprintf(s2, "%d:%d", stop.hour, stop.min);
      sprintf(string, formatTimeRange(s1,s2));
      fprintf(HTML, "%s\n",string);
      fprintf(HTML, " %s %s %d %d\n",
         start.wday, start.month, start.mday, start.year
      );
      fprintf(HTML, "\n<br>\n");
      fprintf(HTML, "</font>\n");
      fprintf(HTML, "<font size=-1>%s</font>\n",s->desc);
      if( strlen(s->directors) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "<font size=-1>Director(s): %s</font>\n",s->directors);
      }
      if( strlen(s->actors) > 0 ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "<font size=-1>Actors: %s</font>\n",s->actors);
      }
      if( s->hdtv ) {
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "<font size=-1>(Available in HDTV)</font>\n");
      }
     }
   }

   // Print out any single recordings with no match
   if( numSingleNoMatch > 0 ) {
      fprintf(HTML, "<br><br><br>\n");
      fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlSingleNoMatchColor);
      fprintf(HTML, "<b>%s</b>\n", CONFIG.htmlSingleNoMatchTitle);
      for( i=0; i<numSingleNoMatch; ++i ) {
         fprintf(HTML, "<hr width=\"100%%\">\n");
         s = singleNoMatches[i];
         utc = guideTimeToUTC(s->Time);
         start = utcToLocalTime(utc);
         utc += 60* (s->Minutes);
         stop = utcToLocalTime(utc);
         sprintf(s1, "%d:%d", start.hour, start.min);
         sprintf(s2, "%d:%d", stop.hour, stop.min);
         sprintf(string, formatTimeRange(s1,s2));
         if( ipCount > 1 ) fprintf(HTML, "%s\n<br>\n", getReplayName(s->ip));
         fprintf(HTML, "%d&nbsp;(%s)&nbsp;&nbsp;&nbsp;<b>%s</b>",
            s->ChannelNum, s->ChannelName, s->Title
         );
         fprintf(HTML, "<br>%s\n",string);
         fprintf(HTML, " %s %s %d %d\n",
            start.wday, start.month, start.mday, start.year
         );
      }
      fprintf(HTML, "</font>\n");
   }

   // Print out any recordings you may miss
   if( NUM_MISSING > 0 ) {
      fprintf(HTML, "<br><br><br>\n");
      fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlMissingColor);
      fprintf(HTML, "<b>%s</b>\n", CONFIG.htmlMissingTitle);
      fprintf(HTML, "<br>\n");
      fprintf(HTML, "<b>Channels Specified: %s</b>\n",
         CONFIG.htmlMissingChannels
      );
      for( i=0; i<NUM_MISSING; ++i ) {
        s = MISSING[i];
        if( ensureUniqueMissed(&unique, &uniqueNum, s, shows, numShows) ) {
         start = utcToLocalTime(s->ustart);
         stop = utcToLocalTime(s->ustop);
         strcpy(repeatStr, "");
         if( s->repeat ) {
            strcpy(repeatStr, " (Repeat)");
         }
         fprintf(HTML, "<hr width=\"100%%\">\n");
         fprintf(HTML, "%s&nbsp;%d (%s)&nbsp;&nbsp;&nbsp;<b>%s%s</b>",
            CONFIG.htmlRecurringChannelTag,
            s->channelNum, s->channel, s->title, repeatStr
         );
         if( strlen(s->subtitle) > 0 ) {
            fprintf(HTML, " - %s", s->subtitle);
         }
         fprintf(HTML, "<br>\n");
         sprintf(s1, "%d:%d", start.hour, start.min);
         sprintf(s2, "%d:%d", stop.hour, stop.min);
         sprintf(string, formatTimeRange(s1,s2));
         fprintf(HTML, "%s\n",string);
         fprintf(HTML, " %s %s %d %d\n",
            start.wday, start.month, start.mday, start.year
         );
         fprintf(HTML, "\n<br>\n");
         fprintf(HTML, "<font size=-1>%s</font>\n",s->desc);
         if( strlen(s->directors) > 0 ) {
           fprintf(HTML, "\n<br>\n");
           fprintf(HTML, "<font size=-1>Director(s): %s</font>\n",s->directors);
         }
         if( strlen(s->actors) > 0 ) {
           fprintf(HTML, "\n<br>\n");
           fprintf(HTML, "<font size=-1>Actors: %s</font>\n",s->actors);
         }
        }
      }
      fprintf(HTML, "</font>\n");
      // Free up memory
      if(unique) {
         for(i=(uniqueNum-1); i>=0; i--) {
            if(unique[i]) free(unique[i]);
         }
         free(unique);
      }
   }

   // Print out empty recurring channels
   if( numEmpty > 0 ) {
      fprintf(HTML, "<br><br><br>\n");
      fprintf(HTML, "<font color=\"%s\">\n", CONFIG.htmlEmptyChannelColor);
      fprintf(HTML, "<b>%s</b>\n", CONFIG.htmlEmptyChannelTitle);
      for( i=0; i<numEmpty; ++i ) {
         fprintf(HTML, "<hr width=\"100%%\">\n");
         s = emptyChannels[i];
         utc = guideTimeToUTC(s->Time);
         start = utcToLocalTime(utc);
         utc += 60* (s->Minutes);
         stop = utcToLocalTime(utc);
         sprintf(s1, "%d:%d", start.hour, start.min);
         sprintf(s2, "%d:%d", stop.hour, stop.min);
         sprintf(string, formatTimeRange(s1,s2));
         if( ipCount > 1 ) fprintf(HTML, "%s\n<br>\n", getReplayName(s->ip));
         fprintf(HTML, "%d&nbsp;(%s)&nbsp;&nbsp;&nbsp;<b>%s</b>",
            s->ChannelNum, s->ChannelName, s->Title
         );
         fprintf(HTML, "<br>%s\n",string);
         fprintf(HTML, " %s %s %d %d\n",
            start.wday, start.month, start.mday, start.year
         );
      }
      fprintf(HTML, "</font>\n");
   }

   // All done with html
   fprintf(HTML, "</body></html>\n");

   // If output was stdout, we don't have to close it
   if((htmlFile[0] != '-') || (htmlFile[1] != 0))
      fclose(HTML);

   // Clean up the allocated memory
   if(days != NULL) {
	   for(i=0; i<numDays; i++)
		   free(days[i]);
	   free(days);
   }

   return(1);
}

char *getReplayName(char ip[]) {
 int i;
   strcpy(_string_,ip);
   if( CONFIG.ipnum > 0 ) {
      for(i=0; i<CONFIG.ipnum; ++i) {
         if( !strcmp(CONFIG.ipmap[i].ip,ip) ) {
            strcpy(_string_,CONFIG.ipmap[i].name);
         }
      }
   }
   return(_string_);
}

// Build unique list of days for all the shows
int getDaysInShows( char ***days, struct Channel **shows, int numShows ) {
   struct Time start;
   char   string[MAXLINE];
   int    i, j, num=0, add;
   int numalloc = 7;

   (*days) = (char **)malloc(sizeof(char *)*numalloc);
   for( i=0; i<numShows; ++i ) {
      start = guideTimeToLocalTime(shows[i]->start);
      sprintf(string, "%s %d (%s)", start.month, start.mday, start.wday);
      add = 1;
      if ( i > 0 ) {
         for( j=0; j<num; ++j ) {
            if( !strcmp(string, (*days)[j]) ) {
               add = 0;
            }
         }
      }
      if( add ) {
         if( num >= numalloc ) {
			 numalloc += 7;
            (*days) = (char **)realloc((*days), sizeof(char *)*(numalloc));
         }
         (*days)[num] = strdup(string);
         num++;
      }
   }
   return(num);
}

char *buildDayLinks( char *day, char **days, int numDays ) {
   static char line[MAXLINE];
   char string[MAXLINE];
   int i;

   strcpy(line,"");
   for( i=0; i<numDays; ++i ) {
      if ( !strcmp(days[i], day) ) {
         sprintf(string, "<A NAME=\"%s\"><b>%s</b></A>", days[i], days[i]);
      } else {
         sprintf(string, "<A HREF=\"#%s\">%s</A>", days[i], days[i]);
      }
	  strcat(line," ");
	  strcat(line,string);
   }
   return(line);
}

char *formatTimeRange( char *start, char *stop ) {
 char s1[128], s2[128];
   strcpy(s1, convertToAmPm(start));
   strcpy(s2, convertToAmPm(stop));
   sprintf(_string_, "%s - %s &nbsp;&nbsp", s1, s2);
   return(_string_);
}

char *convertToAmPm( char *time ) {
   int hour, min;
   char m[3];
   static char string[20];

   sscanf(time, "%d:%d", &hour, &min);
   strcpy(m, "am");
   if (hour>=12) {
      strcpy(m, "pm");
      hour = hour-12;
   }
   if( hour == 0 ) hour = 12;
   sprintf(string, "%02d:%02d %s",hour,min,m);
   return(string);
}

// Reset all members of a program structure
void resetProgStruct( struct prog *prog ) {
   strcpy(prog->start, "");
   strcpy(prog->stop, "");
   prog->ustart = 0;
   prog->ustop = 0;
   prog->channelNum = 0;
   prog->tmsid = 0;
   strcpy(prog->channel, "");
   strcpy(prog->title, "");
   strcpy(prog->subtitle, "");
   strcpy(prog->desc, "");
   strcpy(prog->actors, "");
   strcpy(prog->directors, "");
   prog->repeat = 0;
   prog->hdtv = 0;
}

// Reset all members of a channel structure
void resetChannelStruct( struct Channel *channel ) {
   strcpy(channel->start, "");
   strcpy(channel->stop, "");
   channel->ustart = 0;
   channel->ustop = 0;
   channel->channelNum = 0;
   channel->tmsid = 0;
   strcpy(channel->channel, "");
   strcpy(channel->title, "");
   strcpy(channel->subtitle, "");
   strcpy(channel->desc, "");
   strcpy(channel->actors, "");
   strcpy(channel->directors, "");
   strcpy(channel->SearchKeys, "");
   strcpy(channel->Type, "");
   strcpy(channel->Name, "");
   channel->Minutes = 0;
   strcpy(channel->Keywords, "");
   strcpy(channel->Title, "");
   channel->ChannelNum = 0;
   strcpy(channel->ChannelName, "");
   channel->tmsid = 0;
   strcpy(channel->ip, "");
   strcpy(channel->Time, "");
   strcpy(channel->Days, "");
   channel->Guaranteed = 0;
   channel->PriorityBoost = 0;
   strcpy(channel->ChannelDate, "");
   channel->PaddingBefore = 0;
   channel->PaddingAfter = 0;
   channel->Priority=-1;
   channel->PriorityLoser=-1;
   channel->conflict=0;
   strcpy(channel->ShowFlags, "");
   channel->repeat = 0;
   channel->hdtv = 0;
   channel->firstRun = 0;
   channel->channelID = 0;
   strcpy(channel->CategoryName, "");
   channel->matchcount = 0;
   channel->AtOrAround = 0;
}

// Reset all members of a show structure
void resetShowStruct( struct Show *show ) {
   strcpy(show->ip, "");
   show->ChannelNum = 0;
   strcpy(show->ChannelName, "");
   strcpy(show->Title, "");
   strcpy(show->Description, "");
   strcpy(show->Episode, "");
   strcpy(show->Actors, "");
   strcpy(show->Guests, "");
   strcpy(show->Producers, "");
   strcpy(show->Directors, "");
   strcpy(show->OtherInfo, "");
   show->Minutes = 0;
   show->channelID = 0;
   show->ustart = 0;
   show->ustop = 0;
}

// Substitute weird character for ' in a string
void fixWeirdChars( char *string ) {
int i;
   for( i=0; string[i] != '\0'; ++i ) {
      if( string[i] == '' ) {
         string[i] = '\'';
      }
   }
}

// Fix xml schedule weird '&' representation (all occurrences)
char *fixAmpersans( char *string ) {
	static char s[1000];
   strcpy(s, string);
   while( strstr(s,"&amp;") ) {
      fixAmpersan(s);
   }
   return(s);
}

// Fix xml schedule weird '&' representation (1st occurrence)
void fixAmpersan( char *string ) {
int i, j;
char *ptr;
   // Replace '&amp;' with '&' in a string
   if( (ptr=strstr(string,"&amp;")) ) {
      i=0; j=0;
      // Copy into _string_ until start of ptr
      for( i=0; i<strlen(string)-strlen(ptr); ++i ) {
         _string_[j++] = string[i];
      }

      // Replace weird chars with just '&'
      _string_[j++] = '&';

      // Copy remaining into _string_ after weird chars
      for( i=5; i<strlen(ptr); ++i ) {
         _string_[j++] = ptr[i];
      }
      _string_[j] = '\0';

      strcpy(string, _string_);
   }
}

void copyProgToChan(struct Channel *channel, struct prog *program ) {
   strcpy(channel->start, program->start);
   strcpy(channel->stop, program->stop);
   channel->ustart = program->ustart;
   channel->ustop = program->ustop;
   channel->channelNum = program->channelNum;
   channel->tmsid = program->tmsid;
   strcpy(channel->channel, program->channel);
   strcpy(channel->title, program->title);
   strcpy(channel->subtitle, program->subtitle);
   strcpy(channel->desc, program->desc);
   strcpy(channel->actors, program->actors);
   strcpy(channel->directors, program->directors);
   channel->repeat = program->repeat;
   channel->hdtv = program->hdtv;
}

void printParsedInfo( struct Channel **channels, int num ) {
int i;
   for( i=0; i<num; ++i ) {
      if( channels[i]->Days ) {
          printf("\n");
          printChannelInfo(channels[i]);
      }
   }
}

void printChannelInfo( struct Channel *chan ) {
   struct timeHash *time;

   printf("           IP: '%s'\n", chan->ip);
   printf("         Type: '%s'\n", chan->Type);
   if( strlen(chan->ChannelDate) > 0 ) {
      time = getTimeHash(chan->ChannelDate);
      printf("  ChannelDate: '%02d/%02d/%04d %02d:%02d:%02d'\n",
         time->MM, time->dd, time->yyyy, time->hh, time->mm, time->ss
      );
   }
   if( chan->PriorityBoost )
      printf("PriorityBoost: '%d'\n", chan->PriorityBoost);
   if( strlen(chan->Name) > 0 )
      printf("         Name: '%s'\n", chan->Name);
   if( strlen(chan->Title) > 0 )
      printf("        Title: '%s'\n", chan->Title);
   printf("         Days: '%s'\n", chan->Days);
   if( strlen(chan->ShowFlags) > 0 )
      printf("    ShowFlags: '%s'\n", chan->ShowFlags);
   if( strlen(chan->Keywords) > 0 )
      printf("     Keywords: '%s'\n", chan->Keywords);
   if( strlen(chan->SearchKeys) > 0 )
      printf("   SearchKeys: '%s'\n", chan->SearchKeys);
   if( chan->Minutes )
      printf("      Minutes: '%d'\n", chan->Minutes);
   printf("PaddingBefore: '%d'\n", chan->PaddingBefore);
   printf(" PaddingAfter: '%d'\n", chan->PaddingAfter);
   if( strlen(chan->ChannelName) > 0 )
      printf("      Channel: '%s'\n", chan->ChannelName);
   if(chan->tmsid != 0)
      printf("       TMS ID: '%u'\n", chan->tmsid);
   if( strlen(chan->Time) > 0 ) {
      time = getTimeHash(chan->Time);
      printf("         Time: '%02d/%02d/%04d %02d:%02d:%02d'\n",
         time->MM, time->dd, time->yyyy, time->hh, time->mm, time->ss
      );
   }
   if( chan->firstRun )
      printf("     FirstRun: 'Yes'\n");
   else
      printf("     FirstRun: 'No'\n");
   if( chan->Guaranteed )
      printf("   Guaranteed: '%d'\n", chan->Guaranteed);
}

void printProgramStruct( struct prog *program) {
struct Time start, stop;
   printf("      Channel: '%d %s'\n", program->channelNum, program->channel);
   if(program->tmsid != 0)
      printf("       TMS ID: '%u'\n", program->tmsid);
   printf("        Title: '%s'\n", program->title);
   if( strlen(program->subtitle) > 0 )
      printf("    Sub-Title: '%s'\n", program->subtitle);
   start = utcToLocalTime(program->ustart);
   printf("        Start: '%3s %3s %02d %4d %02d:%02d'\n",
      start.wday, start.month, start.mday, start.year, start.hour, start.min
   );
   stop = utcToLocalTime(program->ustop);
   printf("          End: '%3s %3s %02d %4d %02d:%02d'\n",
      stop.wday, stop.month, stop.mday, stop.year, stop.hour, stop.min
   );
}

void printProgramInfo( struct Channel *program ) {
   struct timeHash *time;
struct Time start, stop;

   printf("           IP: '%s'\n", program->ip);
   printf("      Channel: '%d %s'\n", program->channelNum, program->channel);
   if(program->tmsid != 0)
      printf("       TMS ID: '%u'\n", program->tmsid);
   printf("        Title: '%s'\n", program->title);
   if( strlen(program->subtitle) > 0 )
      printf("    Sub-Title: '%s'\n", program->subtitle);
   start = utcToLocalTime(program->ustart);
   printf("        Start: '%3s %3s %02d %4d %02d:%02d'\n",
      start.wday, start.month, start.mday, start.year, start.hour, start.min
   );
   stop = utcToLocalTime(program->ustop);
   printf("          End: '%3s %3s %02d %4d %02d:%02d'\n",
      stop.wday, stop.month, stop.mday, stop.year, stop.hour, stop.min
   );
   if( strlen(program->ChannelDate) > 0 ) {
      time = getTimeHash(program->ChannelDate);
      printf("  ChannelDate: '%02d/%02d/%04d %02d:%02d:%02d'\n",
         time->MM, time->dd, time->yyyy, time->hh, time->mm, time->ss
      );
   }
   if( program->PriorityBoost )
      printf("PriorityBoost: %d\n", program->PriorityBoost);
   if( strlen(program->desc) > 0 )
      printf("         Desc: '%s'\n", program->desc);
   if( strlen(program->directors) > 0 )
      printf("    Directors: '%s'\n", program->directors);
   if( strlen(program->actors) > 0 )
      printf("       Actors: '%s'\n", program->actors);
   if( program->repeat )
      printf("       Repeat: 'Yes'\n");
   else
      printf("       Repeat: 'No'\n");
   if( program->hdtv )
      printf("         HDTV: 'Yes'\n");
   if( program->Priority )
      printf("     Priority: '%f'\n", program->Priority);
   if( program->AtOrAround)
      printf("   AtOrAround: '%d'\n", program->AtOrAround); 
   if( program->PriorityLoser )
      printf("PriorityLoser: '%d'\n", program->PriorityLoser);
   if( (program->PriorityLoser < 0) || (program->Priority < 0) )
	  printf("             *************** PROGRAM MISSED DURING SCAN **************\n");
   if ( program->conflict ) {
      printf("             ***** CONFLICTING *****\n");
   }
}

// Print program & channel info
void printShowInfo( struct Channel **progs, int num ) {
int i;
   for( i=0; i<num; ++i ) {
      printf("\n");
      printf("#################################\n");
      printProgramInfo(progs[i]);
      printf("\n");
      printChannelInfo(progs[i]);
      printf("#################################\n");
   }
}

void printRecShowInfo( struct Show *show ) {
struct Time start, stop;

   start = utcToLocalTime(show->ustart);
   stop  = utcToLocalTime(show->ustop);

   printf("           IP: '%s'\n", show->ip);
   printf("      Channel: '%d %s'\n", show->ChannelNum, show->ChannelName);
   printf("        Title: '%s'\n", show->Title);
   if( strlen(show->Episode) > 0 )
   printf("      Episode: '%s'\n", show->Episode);
   printf("         Desc: '%s'\n", show->Description);
   if( strlen(show->Actors) > 0 )
      printf("       Actors: '%s'\n", show->Actors);
   if( strlen(show->Guests) > 0 )
      printf("       Guests: '%s'\n", show->Guests);
   if( strlen(show->Producers) > 0 )
      printf("    Producers: '%s'\n", show->Producers);
   if( strlen(show->Directors) > 0 )
      printf("    Directors: '%s'\n", show->Directors);
   if( strlen(show->OtherInfo) > 0 )
      printf("        Other: '%s'\n", show->OtherInfo);
   printf("        Start: '%3s %3s %02d %4d %02d:%02d'\n",
      start.wday, start.month, start.mday, start.year, start.hour, start.min
   );
   printf("          End: '%3s %3s %02d %4d %02d:%02d'\n",
      stop.wday, stop.month, stop.mday, stop.year, stop.hour, stop.min
   );
}

void splitLine(char *line, char ***fields, int *count) {
   static char string[MAXLINE];
   static char *fieldptr[MAXFIELDS];
   int  i, base=0;
   int fieldctr = 0;

   for( i=0; i<MAXLINE; ++i ) {
      string[i] = line[i];
      if( line[i] == '\0' || line[i] == ' ' || line[i] == '\t' || line[i] == '\n' ) {
         string[i] = 0;
         if( string[base] && (fieldctr < MAXFIELDS) )
            fieldptr[fieldctr++] = string+base;
         if(line[i] == '\0') break;
         base = i+1;
      }
   }
   *fields = fieldptr;
   *count = fieldctr;
}

void freeLine(char **fields, int count) {
/*
	int i;
	for(i=(count-1); i>=0; i--) {
		free(fields[i]);
	}
	free(fields);
*/
}

char *joinFields( int start, int stop, char **fields ) {
   int i, j, count=0;
   static char string[MAXLINE];

   for( i=start; i<=stop; ++i ) {
      for( j=0; j<strlen(fields[i]); j++) {
         string[count++] = fields[i][j];
      }
      string[count++] = ' ';
   }
   if ( count > 0 ) {
      string[count-1] = '\0';
   } else {
      string[0] = '\0';
   }   
   return(string);
}

char *parseTimeToGuideTime( char string[] ) {
   int  i, count=0;
   static char parsed[50];

   for( i=0; i<strlen(string); ++i ) {
      if( string[i] != '-' && string[i] != ' ' && string[i] != ':' ) {
         parsed[count++] = string[i];
      }
   }
   parsed[count] = '\0';

   return(parsed);
}

// Break up time into pieces of a structure
// Example: getTimeHash("20020619200000")
//                       yyyyMMddhhmmss
struct timeHash *getTimeHash( char s[] ) {
   static struct timeHash h;
   char yyyy[5], MM[3], dd[3], hh[3], mm[3], ss[3];

   sscanf(s, "%4c%2c%2c%2c%2c%2c", yyyy, MM, dd, hh, mm, ss);
   yyyy[4] = '\0'; MM[2] = '\0'; dd[2] = '\0';
   hh[2] = '\0'; mm[2] = '\0'; ss[2] = '\0';
   //h = (struct timeHash *)malloc(sizeof(struct timeHash));
   h.yyyy = atoi(yyyy);
   if( h.yyyy < 1970 ) {
      h.yyyy = 1970;
   }
   h.MM = atoi(MM);
   h.dd = atoi(dd);
   h.hh = atoi(hh);
   h.mm = atoi(mm);
   h.ss = atoi(ss);

   return(&h);
}

struct Time guideTimeToLocalTime( char guideTime[] ) {
struct Time time;

   time = utcToLocalTime(guideTimeToUTC(guideTime));
   return(time);
}

// Convert time from guide into UTC
// Example: guideTimeToUTC("20020619200000")
//                          yyyyMMddhhmmss
time_t guideTimeToUTC( char s[] ) {
   struct timeHash *h;
   struct tm t;
   time_t time;

   h = getTimeHash(s);
   t.tm_sec =  h->ss;
   t.tm_min =  h->mm;
   t.tm_hour = h->hh;
   t.tm_mday = h->dd;
   t.tm_mon =  h->MM-1;
   t.tm_year = h->yyyy-1900;
   t.tm_isdst = -1;

   time = mktime(&t);

   return(time);
}

// Convert UTC time to local time hash
// Example: utcToLocalTime(time());
struct Time utcToLocalTime(time_t utc) {

struct tm * t;
struct Time T;

	// Remap invalid times to 0
	if(utc == (time_t)-1)
		utc = 0;

   t = localtime(&utc);

   if(t == NULL) {
	   // If localtime fails, fill in anything
	   T.sec   = 0;
	   T.min   = 0;
	   T.hour  = 0;
	   T.mday  = 1;
	   T.year  = 1900;
	   T.yday  = 1;
	   T.isdst = 0;
	   strcpy(T.month, "Jan");
	   strcpy(T.wday, "Sun");
           T.wdayindex = 0;

	   return(T);
   }
   if((t->tm_wday >= 0) && (t->tm_wday < 7))
      T.wdayindex = t->tm_wday;
   else
      T.wdayindex = 0;
   strcpy(T.wday, DayName[T.wdayindex]);

   if((t->tm_mon >= 0) && (t->tm_mon < 12))
      strcpy(T.month, MonthName[t->tm_mon]);
   else
      strcpy(T.month, "Jan");

   T.sec   = t->tm_sec;
   T.min   = t->tm_min;
   T.hour  = t->tm_hour;
   T.mday  = t->tm_mday;
   T.year  = t->tm_year + 1900;
   T.yday  = t->tm_yday;
   T.isdst = t->tm_isdst;

   return(T);
}

// substring matching: find token in string
// considerCase:
//    1 => Case sensitive
//    2 => Look for white space around token
int hasStr( char *string, char *token, int considerCase ) {
   int i, match = 0;
   char s[MAXLINE], t[MAXLINE], *p;

   if( considerCase == 1 ) {
      // Case sensitive comparison
      strcpy(s, string);
      strcpy(t, token);
   } else {
      // Case insensitive comparison
      for( i=0; i<strlen(string); ++i ) {
         s[i] = tolower(string[i]);
      }
      s[i] = '\0';
      for( i=0; i<strlen(token); ++i ) {
         t[i] = tolower(token[i]);
      }
      t[i] = '\0';
   }

   if( (strlen(t) > 0) && (p = strstr(s, t)) ) {
      match = 1;
      if( considerCase == 2 ) {
         // Check for white space around t
         if( p > s ) {
            // Empty space before p
            if( strncmp(p-1," ",1)  ) match = 0;
         }
         if( p+strlen(t)-1 != s+strlen(s)-1 ) {
            // Last char in p should be empty space
            if( !strstr(s, strcat(t," ")) ) match = 0;
         }
      }
   }
   else
      match = 0;

   return(match);
}

int fileStat( char file[], struct stat *status )
{
	if( stat(file, status) == -1 ) {
           return( 0 );
        }
        return( 1 );
}

/*
 * Procedure to check if passed string is a file.
 */
int fileIsFile( char string[] )
{
struct  stat status;

        if( !fileStat(string, &status) )
           return( 0 );

        /* Return True if text is a regular file */
        return( S_ISREG(status.st_mode) && !S_ISDIR(status.st_mode) );
}

/*
 * Procedure to delete a file.
 */
int fileDelete( char file[] )
{
int     result;

        result = unlink( file );
        if( result == 0 )
           return( 1 );
        else
           return( 0 );
}

/*
 * Procedure to return the size of a file in bytes.
 */
int fileSize( char file[] )
{
struct stat statb;

        if( stat(file, &statb) == -1 )
           return( 0 );
        else
           return( statb.st_size );
}

//**********************************************************************************************
// Retrieve the 4K Replay Guide and write to disk
//**********************************************************************************************
void getReplayGuide(char *replayIP, char *binFile)
{
    unsigned char * result;
    unsigned char * timestamp;
    unsigned long size;
    unsigned long r;
    FILE *fp;

    //Tell the user were up to something
    if(!quiet) fprintf(stderr,">> Extracting channel info from '%s' to file '%s'\n", replayIP, binFile);

    r = guide_client_get_snapshot(&result, &timestamp, &size, replayIP, "0");
    if (r) {
        fprintf(stderr, "get_snapshot: error %lx\n", r);
	fprintf(stderr, "Trying to recover using existing file '%s'\n", binFile);
        //exit(-1);
	return;
    }

    //Open the file we will use to store the guide data
    if ((fp = fopen(binFile, "wb")) == NULL) {
        perror("Error: getReplayGuide(): fopen()");
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
        exit(-1);
    }
    fwrite(result, size, 1, fp);
    fclose(fp);
}

static void dump_guide(FILE * fp)
{
    struct replay_guide_snapshot ss;
    unsigned char * buffer;
    unsigned char * p;
    size_t l;
    fseek(fp, 0, SEEK_END);
    l = ftell(fp);
    fseek(fp, 0, SEEK_SET);

    buffer = (unsigned char *)malloc(l * sizeof(unsigned char));
    fread(buffer, l, 1, fp);
    p = buffer;
    parse_replay_guide_snapshot(&p, &ss);
    if(buffer) free(buffer);
    dump_replay_guide_snapshot(&ss);
    free_replay_guide_snapshot(&ss);
}

void makeAsciiGuide(char *binFile, char *asciiFile) {
   FILE *ifp, *ofp;

   //Tell the user were up to something
   if(!quiet) fprintf(stderr,">> Creating ascii channel info file '%s'.\n", asciiFile);

   ifp = fopen(binFile, "rb");
   if (!ifp) {
      fprintf(stderr, "Couldn't open %s\n", binFile);
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
      exit(-1);
   }

   ofp = fopen(asciiFile, "w");
   if (!ofp) {
      fprintf(stderr, "Couldn't write to %s\n", asciiFile);
#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
      exit(-1);
   }

   dump_set_file(ofp);
   dump_group_start(binFile);
   dump_guide(ifp);
   dump_group_end();
   fclose(ifp);
   fclose(ofp);
}

char *buildDumpFile(char *prefix, char *ip)
{
 int i;
   sprintf(_string_, "%s_", prefix);
   for(i=0; i<strlen(ip); ++i) {
      if( ip[i] == '.' || ip[i] == ':' ) {
         sprintf(_string_, "%s_", _string_);
      }
      else {
         sprintf(_string_, "%s%c", _string_, ip[i]);
      }
   }
   return(_string_);
}

// Test hasStr & stripNonAlpha functions used for string matches.
void testSearch() {
char string[100], token[50];

   strcpy(string, "Darma &amp; Greg &amp; Dora");
   printf("weird: '%s'   fixed: '%s'\n", string, fixAmpersans(string));

   strcpy(string, "Will & Grace");
   strcpy(token, "Will Grace");
   printf("stripNonAlpha(%s) = '%s'\n", string, stripNonAlpha(string));
   printf("'%s' has '%s' = %d\n", string, token,
      hasStr(stripNonAlpha(string),token,2)
   );

   strcpy(string, "Liberty's Kids");
   strcpy(token, "Libertys Kids");
   printf("stripNonAlpha(%s) = '%s'\n", string, stripNonAlpha(string));
   printf("'%s' has '%s' = %d\n", string, token,
      hasStr(stripNonAlpha(string),token,2)
   );

   strcpy(string, "Hi-Rev Tuners");
   strcpy(token, "Hi Rev");
   printf("stripNonAlpha(%s) = '%s'\n", string, stripNonAlpha(string));
   printf("'%s' has '%s' = %d\n", string, token,
      hasStr(stripNonAlpha(string),token,2)
   );

   strcpy(string, "a mob-witness/corruption case");
   strcpy(token, "witness corruption");
   printf("stripNonAlpha(%s) = '%s'\n", string, stripNonAlpha(string));
   printf("'%s' has '%s' = %d\n", string, token,
      hasStr(stripNonAlpha(string),token,2)
   );

   strcpy(string, "Costumbres Argentinas");
   strcpy(token, "Argentina");
   printf("stripNonAlpha(%s) = '%s'\n", string, stripNonAlpha(string));
   printf("'%s' has '%s' = %d (expect 0)\n", string, token,
      hasStr(stripNonAlpha(string),token,2)
   );

#ifdef _DEBUG
	   {
		   char buf[10];
		   fputs("Press <ENTER> to exit...",stderr);
		   fgets(buf,sizeof(buf),stdin);
	   }
#endif
   exit(0);
}
