/*==============================================================================
# $Id: replaySchedule.h,v 116.18 2004/12/10 01:17:27 moyekj Exp $
# Copyright (C) 2003 Kevin J. Moye <moyekj@yahoo.com>
#
# Author: Kevin J. Moye
# With contributions by Philip Van Baren
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
==============================================================================*/
#ifndef _DG_H_
#define _DG_H_

#define MAX_IP_NUM 8
#define MAXLINE 1024
#define MAXCHAN 999
#define MAX_SCHEDULE_FILES 5
#define MAXFIELDS 100

#include <time.h>

extern int quiet; // for log message suppression
// for channel guide time coverage
extern time_t minstarttime;
extern time_t maxstarttime;

// Structure for IP -> name mappings
struct ipmap_struct {
   char ip[256];
   char name[256];
};

// Structure containing all user configuration settings
struct config_struct {
   char configFile[256];
   char htmlBackgroundColor[128];
   char htmlForegroundColor[128];
   char htmlConflictLoserColor[128];
   char htmlConflictLoserTitle[128];
   char htmlConflictWinnerColor[128];
   char htmlConflictWinnerTitle[128];
   char htmlSingleNoMatchColor[128];
   char htmlSingleNoMatchTitle[128];
   char htmlMissingColor[128];
   char htmlMissingTitle[128];
   char htmlEmptyChannelColor[128];
   char htmlEmptyChannelTitle[128];
   char htmlMissingChannels[MAXLINE];
   int  htmlDontFlagMissingRepeats;
   char htmlMissingShowIgnore[MAXLINE];
   char htmlSingleChannelTag[56];
   char htmlRecurringChannelTag[56];
   char htmlRecurringChannelAtOrAroundTag[56];
   char htmlThemeChannelTag[56];
   struct ipmap_struct ipmap[MAX_IP_NUM];
   int  ipnum;
   char recordedHtml[256];
} CONFIG;

struct prog {
   char		start[15];
   char		stop[15];
   time_t	ustart;
   time_t	ustop;
   int		channelNum;
   int          tmsid;
   char		channel[15];
   char		title[512];
   char		subtitle[512];
   char		desc[1024];
   char		directors[512];
   char		actors[512];
   int		repeat;
   int		hdtv;
};

struct Channel {
   char		start[15];
   char		stop[15];
   time_t	ustart;
   time_t	ustop;
   int		channelNum;
   char		channel[15];
   char		title[512];
   char		subtitle[512];
   char		desc[1024];
   char		directors[512];
   char		actors[512];
   char		SearchKeys[512];
   char		Type[15];
   char		Name[512];
   int		Minutes;
   char		Keywords[512];
   char		Title[512];
   int		ChannelNum;
   char		ChannelName[50];
   int          tmsid;
   char		Time[50];
   char		Days[50];
   int		Guaranteed;
   char		ChannelDate[50];
   int		PaddingBefore;
   int		PaddingAfter;
   int          PriorityBoost;
   double	Priority;
   int		PriorityLoser;
   int		conflict;
   char		ShowFlags[512];
   int		repeat;
   int		hdtv;
   int		firstRun;
   int          matchcount;
   int          AtOrAround;
   char         ip[256];
   int		channelID;
   char		CategoryName[256];
};

// Structure for already recorded shows
struct Show {
   char         ip[256];
   int		channelID;
   char		CategoryName[256];
   int          ChannelNum;
   char         ChannelName[50];
   char         Title[512];
   char         Description[1024];
   char         Episode[512];
   char         Actors[512];
   char         Guests[512];
   char         Producers[512];
   char         Directors[512];
   char         OtherInfo[512];
   int          Minutes;
   time_t       ustart;
   time_t       ustop;
};

struct chanMap {
   char name[50];
   char mapped[50];
};

struct timeHash {
   int	yyyy;
   int	MM;
   int	dd;
   int	hh;
   int	mm;
   int	ss;
};

struct Time {
   int  sec;
   int  min;
   int  hour;
   int  mday;
   char month[4];
   int  year;
   char wday[4];
   int  wdayindex;
   int  yday;
   int  isdst;
};

struct stopTime {
   char   channel[256];
   char   stop[24];
};

void useage();
int setupConfiguration();
int convertReplayGuide(
   char replayIP[], struct Channel ***channels, int *num,
   int **CategoryIndex, char ***CategoryName, int *numCategory,
   int *cnum, int *catnum, int *indexes, int *indexalloc,
   struct Show ***shows, int *numS, int *snum
);
int parseReplayGuide(
   char replayIP[], char asciiGuideFile[], struct Channel ***channels, int *num,
   int **CategoryIndex, char ***CategoryName, int *numCategory,
   int *cnum, int *catnum, int *indexes, int *indexalloc,
   struct Show ***shows, int *numS, int *snum
);
int computeMissingStopTimes(char xmltvScheduleFile[], struct stopTime **stopTimes);
int extractRecordingSchedule(
   char xmltvScheduleFile[],
   struct Channel **channels, int numChan,
   struct Channel ***progs, int *pnum, int *numProg,
   struct stopTime *stopTimes
);
int extractRecordingScheduleDD(
   char xmltvScheduleFile[],
   struct Channel **channels, int numChan,
   struct Channel ***progs, int *pnum, int *numProg,
   struct stopTime *stopTimes
);
int parseChannelGuide(
   char ChannelGuideScheduleFile[], struct Channel **channels, int numChan,
   struct Channel ***progs, int *pnum, int *numProg
);
int checkForMissingSingleRecordings(
   struct Channel **channels, int numChan,
   struct Channel ***singleNoMatches, int *numSingleNoMatch
);
int checkForEmptyRecurringChannels(
   struct Channel **channels, int numChan,
   struct Channel ***emptyChannels, int *numEmpty
);
struct Channel *compareProgramToChannels(
   struct prog *progList,
   struct Channel **channels,
   int    numChan
);
struct Channel *copyMatchedProgram(struct prog *program, struct Channel *channel);
int flagMissingHandler( struct Channel *chan, struct prog *program );
int flagThisChannel(char name[]);
int ignoreThisMissedShow(char name[], int repeat);
int compareSingle(struct Channel *chan, struct prog *program);
int compareRecurring(struct Channel *chan, struct prog *program);
int compareTheme(struct Channel *chan, struct prog *program);
void buildChannelMap(char mapFile[]);
char *channelMap(char *channel);
void checkForConflicts( struct Channel ***recordings, int numProg );
void sortProgs(
   struct Channel ***sorted, struct Channel ***progs, int numProg
);
int createXmlSchedule(
   char *htmlFile, char *replayIP, struct Channel **shows, int numShows,
   int *CategoryIndex, char **CategoryName, int numCategory, int noTheme
);
int createHtmlSchedule(
   char htmlFile[], char replayIP[MAX_IP_NUM][256], int ipCount, struct Channel **shows, int numShows,
   int  noTheme, struct Channel **singleNoMatches, int numSingleNoMatch,
   struct Channel **emptyChannels, int numEmpty
);
int createRecordedSchedule(
   char htmlFile[], char replayIP[MAX_IP_NUM][256], int ipCount,
   struct Show **shows, int numShows,
   struct Channel **channels, int numChan
);
char *getReplayName(char ip[]);
int getDaysInShows( char ***days, struct Channel **shows, int numShows );
char *buildDayLinks( char *string, char **days, int numDays );
char *formatTimeRange( char *start, char *stop );
char *convertToAmPm( char *time );
void printParsedInfo( struct Channel **channels, int num );
void printChannelInfo( struct Channel *chan );
void printProgramInfo( struct Channel *program );
void printProgramStruct( struct prog *program );
void printShowInfo( struct Channel **progs, int num );
void printRecShowInfo( struct Show *show );
void resetProgStruct( struct prog *prog );
void resetChannelStruct( struct Channel *channel );
void resetShowStruct( struct Show *show );
void copyProgToChan(struct Channel *channel, struct prog *program );
void splitLine(char *line, char ***fields, int *count);
void freeLine(char **fields, int count);
char *joinFields( int start, int stop, char **fields );
char *parseTimeToGuideTime( char string[] );
void fixWeirdChars( char *string );
char *fixAmpersans( char *string );
void fixAmpersan( char *string );
int ensureUniqueProgram(
   char ***unique, int *uniqueNum, struct Channel *match
);
int ensureUniqueMissed(
   char ***unique, int *uniqueNum, struct Channel *match,
   struct Channel **shows, int numShows
);
int hasStr( char *string, char *token, int considerCase );
char *stripNonAlpha(char *string);
int strEqualStripped(char *s1, char *s2);
int guideMatchStr( char *string, char *token );
int fileStat( char file[], struct stat *status );
int fileIsFile( char string[] );
int fileDelete( char file[] );
int fileSize( char file[] );
void getReplayGuide(char *replayIP, char *binFile);
void makeAsciiGuide(char *binFile, char *asciiFile);
struct timeHash *getTimeHash( char s[] );
struct Time guideTimeToLocalTime( char guideTime[] );
time_t guideTimeToUTC( char s[] );
struct Time utcToLocalTime(time_t utc);

unsigned long guide_client_get_snapshot(
   unsigned char ** presult, unsigned char ** ptimestamp,
   unsigned long * psize, const char * address,
   const char * cur_timestamp
);

void testSearch();

char *buildDumpFile(char *prefix, char *ip);

int
parseXMLFile(
      char schedule_6412[], struct Channel ***channels, int *numC,
      char ***CategoryName, int *numCategories, int *cnum, int *catnum
);

#endif
